import numpy as np
import pandas as pd


def event_filter_by_errorbar_predicted_value_ratio(working_data_df, model_type, piE_type, precision_threshold=0.3):
    """
    Filter events based on the ratio of predicted errorbar to predicted value.
    :param working_data_df:
    :param model_type:
    :param piE_type:
    :param precision_threshold:
    :return:
    """
    approved_df = working_data_df[working_data_df[
                                      f'{model_type}_{piE_type}_absolute_predicted_errorbar_predicted_value_ratio'] < precision_threshold].copy()
    return approved_df


def event_filter_by_relative_accuracy(working_data_df, model_type, piE_type, accuracy_threshold=0.3):
    """
    Filter events based on the relative difference from the true value.
    :param working_data_df:
    :param model_type:
    :param piE_type:
    :param accuracy_threshold:
    :return:
    """
    # Metrics
    approved_df = working_data_df[
        working_data_df[f'{model_type}_{piE_type}_relative_difference_from_true_value'] < accuracy_threshold].copy()
    return approved_df


def combined_filtering_creteria(approved_precision_df, approved_accuracy_df):
    """
    Combine the two filters to get the final approved events.
    :param approved_precision_df:
    :param approved_accuracy_df:
    :return:
    """
    # Combine the two filters
    combined_approved_df = approved_precision_df[approved_precision_df.index.isin(approved_accuracy_df.index)].copy()
    return combined_approved_df


def main(statistics_data_path_, general_path_outputs_, precision_threshold=0.3, accuracy_threshold=0.3):
    """
    Main function to filter events based on the statistical analysis.
    :param statistics_data_path_:
    :param general_path_outputs_:
    :return:
    """
    working_data = pd.read_csv(
        statistics_data_path_,
        index_col=['event_name'])
    # piEE is the main goal
    piE_type = 'piEE'

    # LX GOOD FOR piEE
    model_type = 'LX'
    lx_approved_precision_df = event_filter_by_errorbar_predicted_value_ratio(working_data, model_type, piE_type,
                                                                              precision_threshold)
    lx_approved_precision_df.to_csv(f'{general_path_outputs_}/'
                                    'lx_precision_0.3_df.csv')
    lx_approved_accuracy_df = event_filter_by_relative_accuracy(working_data, model_type, piE_type,
                                                                accuracy_threshold)
    lx_approved_accuracy_df.to_csv(f'{general_path_outputs_}/'
                                   'lx_accuracy_0.3_df.csv')
    lx_combined_approved_df = combined_filtering_creteria(lx_approved_precision_df, lx_approved_accuracy_df)
    lx_combined_approved_df.to_csv(f'{general_path_outputs_}/'
                                   'lx_combined_0.3_df.csv')

    # LO GOOD FOR piEE
    model_type = 'LO'
    lo_approved_precision_df = event_filter_by_errorbar_predicted_value_ratio(working_data, model_type, piE_type,
                                                                              precision_threshold)
    lo_approved_precision_df.to_csv(f'{general_path_outputs_}/'
                                    'lo_precision_0.3_df.csv')
    lo_approved_accuracy_df = event_filter_by_relative_accuracy(working_data, model_type, piE_type,
                                                                accuracy_threshold)
    lo_approved_accuracy_df.to_csv(f'{general_path_outputs_}/'
                                   'lo_accuracy_0.3_df.csv')
    lo_combined_approved_df = combined_filtering_creteria(lo_approved_precision_df, lo_approved_accuracy_df)
    lo_combined_approved_df.to_csv(f'{general_path_outputs_}/'
                                   'lo_combined_0.3_df.csv')

    # LX and LO GOOD FOR piEE
    lx_and_lo_precision_df = lx_approved_precision_df[
        lx_approved_precision_df.index.isin(lo_approved_precision_df.index)].copy()
    lx_and_lo_precision_df.to_csv(f'{general_path_outputs_}/'
                                  'lx_and_lo_precision_0.3_df.csv')
    lx_and_lo_accuracy_df = lx_approved_accuracy_df[
        lx_approved_accuracy_df.index.isin(lo_approved_accuracy_df.index)].copy()
    lx_and_lo_accuracy_df.to_csv(f'{general_path_outputs_}/'
                                 'lx_and_lo_accuracy_0.3_df.csv')
    lx_and_lo_combined_df = lx_combined_approved_df[
        lx_combined_approved_df.index.isin(lo_combined_approved_df.index)].copy()
    lx_and_lo_combined_df.to_csv(f'{general_path_outputs_}/'
                                 'lx_and_lo_combined_0.3_df.csv')
    print()


if __name__ == '__main__':
    general_path_outputs = '/RTModel_plots/Notebook_statiscs_by_cuts'
    statistics_data_path = '/analysis/rtmodelv2.4_statistics.csv'
    main(statistics_data_path, general_path_outputs)



from jasmine.classes_and_files_reader.RTModel_results_cls import ModelResults
from jasmine.classes_and_files_reader.datachallenge_lightcurve_cls import LightcurveEventDataChallenge
from jasmine.classes_and_files_reader.rtmodel_template_cls import RTModelTemplateForBinaryLightCurve

__all__ = ['ModelResults', 'LightcurveEventDataChallenge', 'RTModelTemplateForBinaryLightCurve']





'''
Linear limb darkening coefficients in various bands
Coefficients for moa2r, moa2V, ogleI, ogleV, and cfht_I are the same values used by the Goddard Microlensing group
Coefficients for W149, W146, Z087, R062, F184 and K213 were derived from Claret & Bloemen 2011
https://www.aanda.org/articles/aa/full_html/2011/05/aa16451-11/aa16451-11.html
'''

moa2r = 0.7259
moa2V = 0.6730 
ogleI = 0.6155
ogleV = 0.6730
cfht_I = 0.5090
W149 = 0.3300
W146 = 0.3300 # just a median over K and H bands.
Z087 = 0.4534
R062 = 0.5987
F184 = 0.2914 # JB: I used H-Band. Not quite right, but closest match
K213 = 0.2517




import numpy as np
from jasmine.caustics.caustic_creator_cls import Caustic


class SourceTrajectoryCausticCrossing:
    """
    This class creates the source trajectory of the lensed source star if crossing a caustic.
    y = ax + b

    """

    def __init__(self, separation_s, mass_ratio_q,
                 inclination_of_source_trajectory,
                 index_for_caustic_data_points,
                 caustic_number_of_data_points=100,
                 source_trajectory_number_of_data_points=100):
        self.separation_s = separation_s
        self.mass_ratio_q = mass_ratio_q
        self.inclination_of_source_trajectory = inclination_of_source_trajectory
        self.index_for_caustic_data_points = index_for_caustic_data_points
        self.caustic_number_of_data_points = caustic_number_of_data_points
        self.source_trajectory_number_of_data_points = source_trajectory_number_of_data_points
        self.caustic = Caustic(separation_s=separation_s,
                               mass_ratio_q=mass_ratio_q,
                               number_of_data_points=caustic_number_of_data_points)
        self.starting_point_x = self.caustic.horizontal[self.index_for_caustic_data_points]
        self.starting_point_y = self.caustic.vertical[self.index_for_caustic_data_points]
        self.offset_of_source_trajectory = offset_for_line_through_point(self.starting_point_x,
                                                                         self.starting_point_y,
                                                                         self.inclination_of_source_trajectory)
        self.source_x = np.linspace(-3, 3, self.source_trajectory_number_of_data_points)
        self.source_y = self.inclination_of_source_trajectory * self.source_x + self.offset_of_source_trajectory

    # caustic.define_topology()


def offset_for_line_through_point(x0, y0, a):
    """
    Returns a function of a line y = ax + b that passes through the point (x1, y1).

    Parameters:
    x0 (float): The x-coordinate of the starting/crossing point.
    y0 (float): The y-coordinate of the starting/crossing point.
    a (float): The slope of the line.
    Returns:
    b (float): offset b
    """
    # Calculate the y-intercept b using the point (x1, y1)
    b = y0 - a * x0
    return b


if __name__ == '__main__':
    source_trajectory = SourceTrajectoryCausticCrossing(separation_s=0.8,
                                                        mass_ratio_q=0.3,
                                                        inclination_of_source_trajectory=0.5,
                                                        index_for_caustic_data_points=50,
                                                        caustic_number_of_data_points=100,
                                                        source_trajectory_number_of_data_points=100)
    import matplotlib.pyplot as plt
    plt.scatter(source_trajectory.caustic.horizontal, source_trajectory.caustic.vertical, s=0.1)
    plt.scatter(source_trajectory.starting_point_x, source_trajectory.starting_point_y, color='red')
    plt.plot(source_trajectory.source_x, source_trajectory.source_y, c='black')
    plt.show()


"""Plots for histogram and cumulative distribution function of the data."""

import matplotlib.pyplot as plt
import numpy as np
import matplotlib.pyplot as plt


def cumulative_distribution_presentation_plotter(working_data_, string_LX, string_LO,
                                    title=None, xmax_linear=40, what_you_plotting='Planet Mass', working_data_2_=None):
    # Scores in ascending order:
    LX_standardized_residual_order = working_data_.sort_values(f'{string_LX}')[f'{string_LX}']
    if working_data_2_ is not None:
        LO_standardized_residual_order = working_data_2_.sort_values(f'{string_LO}')[f'{string_LO}']
    else:
        LO_standardized_residual_order = working_data_.sort_values(f'{string_LO}')[f'{string_LO}']

    # Inserting 0 and 1s
    LX_standardized_residual_order = np.insert(LX_standardized_residual_order, 0, 0)
    LX_standardized_residual_order = np.insert(LX_standardized_residual_order, len(LX_standardized_residual_order), 1)
    LO_standardized_residual_order = np.insert(LO_standardized_residual_order, 0, 0)
    LO_standardized_residual_order = np.insert(LO_standardized_residual_order, len(LO_standardized_residual_order), 1)

    # Defining CDF values
    step_LX = 1 / (len(LX_standardized_residual_order) - 1)
    LX_cdf = np.arange(0, 1 + step_LX / 2, step_LX)
    step_LO = 1 / (len(LO_standardized_residual_order) - 1)
    LO_cdf = np.arange(0, 1 + step_LO / 2, step_LO)

    # Create subplots for two different types of plots
    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(14, 6))

    # Logarithmic scale plot
    ax1.step(LX_standardized_residual_order, LX_cdf, label=f'Parallax: {len(LX_standardized_residual_order) - 2}')
    ax1.step(LO_standardized_residual_order, LO_cdf,
             label=f'Parallax + Orbital Motion: {len(LO_standardized_residual_order) - 2}', color='orange')
    ax1.set_xscale('log')
    ax1.set_ylim([0, 1 + 1e-3])
    ax1.set_xlabel(fr'{what_you_plotting}')
    ax1.set_ylabel('Cumulative Distribution')
    ax1.set_title(f'LOG SCALE | {title}' if title else 'Cumulative Distribution - Log Scale')
    ax1.legend()

    # Linear scale plot with x-range limited to xmax_linear=40 default
    ax2.step(LX_standardized_residual_order, LX_cdf, label=f'Parallax: {len(LX_standardized_residual_order) - 2}')
    ax2.step(LO_standardized_residual_order, LO_cdf,
             label=f'Parallax + Orbital Motion: {len(LO_standardized_residual_order) - 2}', color='orange')
    ax2.set_xlim([-1e-5, xmax_linear])
    ax2.set_ylim([-1e-5, 1 + 1e-3])
    ax2.set_xlabel(fr'{what_you_plotting}')
    ax2.set_ylabel('Cumulative Distribution')
    ax2.set_title(f'LINEAR SCALE + ZOOM | {title}' if title else 'ZOOM - Cumulative Distribution - Linear Scale')
    ax2.legend()

    # Show the plots
    plt.tight_layout()
    plt.savefig(f"{str({what_you_plotting}).replace(' ','_').lower()}.png", dpi=300)


def histogram_presentation_plotter(working_data_, string_LX, string_LO,
                                   title=None, xmax_linear=40, what_you_plotting='Planet Mass', bins=30, working_data_2_=None):
    # Extract data for histograms
    LX_data = working_data_[string_LX].dropna()
    if working_data_2_ is not None:
        LO_data = working_data_2_[string_LO].dropna()
    else:
        LO_data = working_data_[string_LO].dropna()

    # Create subplots for two different types of plots
    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(14, 4))

    # Logarithmic scale plot
    ax1.hist(LX_data, bins=bins, alpha=0.4, label=f'Parallax: {len(LX_data)}', log=True)
    ax1.hist(LO_data, bins=bins, alpha=0.4, label=f'Parallax + Orbital Motion: {len(LO_data)}', log=True)
    ax1.set_xscale('log')
    ax1.set_xlabel(what_you_plotting)
    ax1.set_ylabel('Count')
    ax1.set_title(f'LOG SCALE | {title}' if title else 'Histogram - Log Scale')
    ax1.legend()

    # Linear scale plot with x-range limited to xmax_linear
    ax2.hist(LX_data, bins=bins, alpha=0.4, label=f'Parallax: {len(LX_data)}')
    ax2.hist(LO_data, bins=bins, alpha=0.4, label=f'Parallax + Orbital Motion: {len(LO_data)}')
    ax2.set_xlim([-xmax_linear, xmax_linear])
    ax2.set_xlabel(what_you_plotting)
    ax2.set_ylabel('Count')
    ax2.set_title(f'LINEAR SCALE + ZOOM | {title}' if title else 'Histogram - Linear Scale')
    ax2.legend()

    # Show the plots
    plt.tight_layout()
    plt.savefig(f"{what_you_plotting.replace(' ', '_').lower()}_histogram.png", dpi=300)
    plt.show()





from dataclasses import dataclass

import jasmine.classes_and_files_reader.new_gullsrges_reader as grr
import pandas as pd


class LightcurveEventGULLSRGES_NameBased:
    def __init__(self, lightcurve_name, data_folder='../data/gulls_orbital_motion_extracted/',
                 master_file_path='../data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan.sample.csv'):
        # This is for the new team-simulations release (from 2024Nov).
        master_df = pd.read_csv(master_file_path)
        self.lightcurve_name = lightcurve_name  #e.g. OMPLDG_croin_cassan_0_642_57.det.lc
        self.lightcurve_master = master_df[master_df.lcname == self.lightcurve_name]
        self.event_info = EventInformation(self.lightcurve_master)
        self.sample_folder = data_folder
        self.light_curve_path = f"{self.sample_folder}/{self.lightcurve_name}"
        self.source = SourceStarProperties(self.lightcurve_master)
        self.weights_and_flags = WeightAndFlags(self.lightcurve_master)
        self.lens = LensStarProperties(self.lightcurve_master)
        self.microlensing = MicrolensingParameters(self.lightcurve_master)
        self.planet = PlanetProperties(self.lightcurve_master)

    def lightcurve_data(self):
        """
        This function reads the lightcurve data and returns a tuple of pandas dataframes with the columns
        :param folder_to_be_created_path_:
        :return:
        """
        lightcurve_df_list = grr.lightcurve_data_reader(self.event_info.event_subrun,
                                                        self.event_info.event_field,
                                                        self.event_info.event_id,
                                                        folder_path_=self.sample_folder,
                                                        include_ephem=True)
        return lightcurve_df_list

    def microlensing_signal(self):
        """
        This function reads the microlensing signal data and returns a tuple of pandas dataframes with the columns
        :param folder_to_be_created_path_:
        :return:
        """
        microlensing_df_list = grr.microlensing_signal_reader(self.event_info.event_subrun,
                                                              self.event_info.event_field,
                                                              self.event_info.event_id,
                                                              folder_path_=self.sample_folder,
                                                              include_ephem=True)
        return microlensing_df_list

    def whole_lightcurve(self):
        """
        This function reads the whole lightcurve data and returns a pandas dataframe with the columns
        :param folder_to_be_created_path_:
        :return:
        """
        lightcurve_df = grr.whole_columns_lightcurve_reader(self.event_info.event_subrun,
                                                            self.event_info.event_field,
                                                            self.event_info.event_id,
                                                            folder_path_=self.sample_folder,
                                                            include_ephem=True)
        return lightcurve_df


@dataclass
class EventInformation:
    """
    Field and Coordinate Information: This includes the event's index, subrun number, field,
    galactic coordinates (l, b), and right ascension and declination (ra, dec).
    """
    event_id: int
    event_subrun: int
    event_field: int
    event_galactic_longitude__deg: float
    event_galactic_latitude__deg: float
    event_right_ascension__deg: float
    event_declination__deg: float

    # event_type: str
    # lc_root: str
    # data_challenge_lc_number: int

    def __init__(self, lightcurve_master: pd.DataFrame):
        self.event_id = lightcurve_master['EventID'].values[0]
        self.event_subrun = lightcurve_master['SubRun'].values[0]
        self.event_field = lightcurve_master['Field'].values[0]
        self.event_galactic_longitude__deg = lightcurve_master['galactic_l'].values[0]
        self.event_galactic_latitude__deg = lightcurve_master['galactic_b'].values[0]
        self.event_right_ascension__deg = lightcurve_master['ra_deg'].values[0]
        self.event_declination__deg = lightcurve_master['dec_deg'].values[0]


@dataclass
class SourceStarProperties:
    """
    Source Star Properties: Details about the source star, such as its ID, distance (Ds), radius (Rs),
    proper motion (smu_l, smu_b), age, class (scl).
    Magnitudes: The magnitudes of the source in various filters (e.g., J, F087, H, W149, W169).
    """
    source_id: int
    source_distance__kpc: float
    source_radius__rsun: float
    source_age: int
    source_class: int
    source_mass__msun: float
    source_effective_temperature__k: float
    source_surface_gravity: float
    source_metalicity: float
    source_alpha_Fe: float
    source_pop: int
    source_initial_mass__msun: float
    source_bolometric_magnitude: float
    source_Av: float

    source_l: float
    source_b: float
    source_ra: float
    source_dec: float
    source_x: float
    source_y: float
    source_z: float
    source_proper_motion_l_1masyr: float
    source_proper_motion_b_1masyr: float

    source_magnitude_R062__mag: float
    source_magnitude_Z087__mag: float
    source_magnitude_Y106__mag: float
    source_magnitude_J129__mag: float
    source_magnitude_W146__mag: float
    source_magnitude_H158__mag: float
    source_magnitude_F184__mag: float
    source_magnitude_K213__mag: float
    source_magnitude_2MASS_Ks__mag: float
    source_magnitude_2MASS_J__mag: float
    source_magnitude_2MASS_H__mag: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        self.source_id = lightcurve_master['SourceID'].values[0]

        self.source_distance__kpc = lightcurve_master['Source_Dist'].values[0]
        self.source_radius__rsun = lightcurve_master['Source_Radius'].values[0]
        self.source_age = lightcurve_master['Source_age'].values[0]
        self.source_class = lightcurve_master['Source_CL'].values[0]
        # self.source_type = lightcurve_master['styp'].values[0]
        self.source_mass__msun = lightcurve_master['Source_Mass'].values[0]
        self.source_effective_temperature__k = lightcurve_master['Source_Teff'].values[0]
        self.source_surface_gravity = lightcurve_master['Source_logg'].values[0]
        self.source_metalicity = lightcurve_master['Source_Fe_H'].values[0]
        self.source_alpha_Fe = lightcurve_master['Source_alpha_Fe'].values[0]
        self.source_pop = lightcurve_master['Source_pop'].values[0]
        self.source_initial_mass__msun = lightcurve_master['Source_iMass'].values[0]
        self.source_bolometric_magnitude = lightcurve_master['Source_Mbol'].values[0]
        self.source_Av = lightcurve_master['Source_Av'].values[0]
        self.source_l = lightcurve_master['Source_l'].values[0]
        self.source_b = lightcurve_master['Source_l'].values[0]
        self.source_ra = lightcurve_master['Source_RA20000'].values[0]
        self.source_dec = lightcurve_master['Source_DEC20000'].values[0]
        self.source_x = lightcurve_master['Source_x'].values[0]
        self.source_y = lightcurve_master['Source_y'].values[0]
        self.source_z = lightcurve_master['Source_z'].values[0]
        self.source_proper_motion_l_1masyr = lightcurve_master['Source_mul'].values[0]
        self.source_proper_motion_b_1masyr = lightcurve_master['Source_mub'].values[0]

        self.source_magnitude_R062__mag = lightcurve_master['Source_R062'].values[0]
        self.source_magnitude_Z087__mag = lightcurve_master['Source_Z087'].values[0]
        self.source_magnitude_Y106__mag = lightcurve_master['Source_Y106'].values[0]
        self.source_magnitude_J129__mag = lightcurve_master['Source_J129'].values[0]
        self.source_magnitude_W146__mag = lightcurve_master['Source_W146'].values[0]
        self.source_magnitude_H158__mag = lightcurve_master['Source_H158'].values[0]
        self.source_magnitude_F184__mag = lightcurve_master['Source_F184'].values[0]
        self.source_magnitude_K213__mag = lightcurve_master['Source_K213'].values[0]
        self.source_magnitude_2MASS_Ks__mag = lightcurve_master['Source_2MASS_Ks'].values[0]
        self.source_magnitude_2MASS_J__mag = lightcurve_master['Source_2MASS_J'].values[0]
        self.source_magnitude_2MASS_H__mag = lightcurve_master['Source_2MASS_H'].values[0]


@dataclass
class LensStarProperties:
    """
    Lens Star Properties: Information on the lens star, including its ID, distance (Dl), mass (Ml),
    proper motion (lmu_l, lmu_b), age, class (lcl), type (ltyp), bolometric magnitude (lMbol),
    effective temperature (lTeff), surface gravity (Llogg), and radius (Rl).
    """
    lens_system_mass__msun: float
    lens_initial_mass__msun: float
    lens_distance__kpc: float
    lens_radius__rsun: float

    lens_l: float
    lens_b: float
    lens_ra: float
    lens_dec: float
    lens_x: float
    lens_y: float
    lens_z: float
    lens_proper_motion_l_1masyr: float
    lens_proper_motion_b_1masyr: float

    lens_id: int
    lens_age: int
    lens_class: int
    lens_metalicity: float
    lens_alpha_Fe: float
    lens_pop: int
    lens_Av: float

    # lens_type: float
    lens_surface_gravity: float
    lens_bolometric_magnitude: float
    lens_effective_temperature__k: float

    lens_magnitude_R062__mag: float
    lens_magnitude_Z087__mag: float
    lens_magnitude_Y106__mag: float
    lens_magnitude_J129__mag: float
    lens_magnitude_W146__mag: float
    lens_magnitude_H158__mag: float
    lens_magnitude_F184__mag: float
    lens_magnitude_K213__mag: float
    lens_magnitude_2MASS_Ks__mag: float
    lens_magnitude_2MASS_J__mag: float
    lens_magnitude_2MASS_H__mag: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        # MASSES
        # Ml: mass of the lens
        self.lens_system_mass__msun = lightcurve_master['Lens_Mass'].values[0]
        self.lens_initial_mass__msun = lightcurve_master['Lens_iMass'].values[0]

        # Dl: distance to the lens
        self.lens_distance__kpc = lightcurve_master['Lens_Dist'].values[0]
        # Radius
        self.lens_radius__rsun = lightcurve_master['Lens_Radius'].values[0]

        # Position and Motion

        self.lens_l = lightcurve_master['Lens_l'].values[0]
        self.lens_b = lightcurve_master['Lens_b'].values[0]
        self.lens_ra = lightcurve_master['Lens_RA20000'].values[0]
        self.lens_dec = lightcurve_master['Lens_DEC20000'].values[0]
        self.lens_x = lightcurve_master['Lens_x'].values[0]
        self.lens_y = lightcurve_master['Lens_y'].values[0]
        self.lens_z = lightcurve_master['Lens_z'].values[0]
        self.lens_proper_motion_l_1masyr = lightcurve_master['Lens_mul'].values[0]
        self.lens_proper_motion_b_1masyr = lightcurve_master['Lens_mub'].values[0]

        # Other lens parameters
        self.lens_id = lightcurve_master['LensID'].values[0]
        self.lens_age = lightcurve_master['Lens_age'].values[0]
        self.lens_class = lightcurve_master['Lens_CL'].values[0]
        # self.lens_type = lightcurve_master['ltyp'].values[0]

        self.lens_surface_gravity = lightcurve_master['Lens_logg'].values[0]
        self.lens_bolometric_magnitude = lightcurve_master['Lens_Mbol'].values[0]
        self.lens_effective_temperature__k = lightcurve_master['Lens_Teff'].values[0]
        self.lens_metalicity = lightcurve_master['Lens_Fe_H'].values[0]
        self.lens_alpha_Fe = lightcurve_master['Lens_alpha_Fe'].values[0]
        self.lens_pop = lightcurve_master['Lens_pop'].values[0]
        self.lens_Av = lightcurve_master['Lens_Av'].values[0]

        # Magnitudes: The magnitudes of the lens in various filters (e.g., J, F087, H, W149, W169).
        self.lens_magnitude_R062__mag = lightcurve_master['Lens_R062'].values[0]
        self.lens_magnitude_Z087__mag = lightcurve_master['Lens_Z087'].values[0]
        self.lens_magnitude_Y106__mag = lightcurve_master['Lens_Y106'].values[0]
        self.lens_magnitude_J129__mag = lightcurve_master['Lens_J129'].values[0]
        self.lens_magnitude_W146__mag = lightcurve_master['Lens_W146'].values[0]
        self.lens_magnitude_H158__mag = lightcurve_master['Lens_H158'].values[0]
        self.lens_magnitude_F184__mag = lightcurve_master['Lens_F184'].values[0]
        self.lens_magnitude_K213__mag = lightcurve_master['Lens_K213'].values[0]
        self.lens_magnitude_2MASS_Ks__mag = lightcurve_master['Lens_2MASS_Ks'].values[0]
        self.lens_magnitude_2MASS_J__mag = lightcurve_master['Lens_2MASS_J'].values[0]
        self.lens_magnitude_2MASS_H__mag = lightcurve_master['Lens_2MASS_H'].values[0]


@dataclass
class MicrolensingParameters:
    """
    Microlensing Event Properties: Parameters such as the impact parameter (u0), event angle (alpha),
    time of maximum magnification (t0), Einstein crossing time (tE), Einstein radius (rE), theta_E (thE),
    parallax effect (piE), source radius in Einstein radii (rhos), relative proper motion (murel),
    transverse velocity (vt), and shear (gamma).

    Blending Flux (fs): The blending parameter, which is crucial for determining the combined
    flux from the source, lens, and any additional blended light.
    """
    microlensing_impact_parameter_u0: float
    microlensing_event_angle_alpha__deg: float
    microlensing_time_of_peak_t0__days: float
    # What are these
    microlensing_tcroin_days: float
    microlensing_ucroin: float
    microlensing_rcroin: float
    ### heliocentric and reference frame tE (in Sun and Roman Frames)
    microlensing_einstein_crossing_time_tE_ref__days: float
    microlensing_einstein_crossing_time_tE_helio__days: float
    microlensing_einstein_radius_rE__AU: float
    microlensing_einstein_angular_radius_thetaE__mas: float
    microlensing_source_radius_in_einstein_radii_rho: float
    # Parallax
    microlensing_parallax_effect_piE: float
    microlensing_parallax_effect_piE_East: float
    microlensing_parallax_effect_piE_North: float
    microlensing_parallax_effect_piE_par: float
    microlensing_parallax_effect_piE_rp: float
    # Heliocentric Relative Proper Motion
    microlensing_relative_proper_motion_murel_helio__1masyr: float
    microlensing_relative_proper_motion_murel_alpha_helio___1masyr: float
    microlensing_relative_proper_motion_murel_delta_helio___1masyr: float
    microlensing_relative_proper_motion_murel_l_helio___1masyr: float
    microlensing_relative_proper_motion_murel_b_helio___1masyr: float
    microlensing_relative_proper_motion_murel_lambda_helio___1masyr: float
    # Reference Frame Relative Proper Motion (Roman)
    microlensing_relative_proper_motion_murel_ref__1masyr: float
    microlensing_relative_proper_motion_murel_beta_ref___1masyr: float
    microlensing_relative_proper_motion_murel_ref__1masyr: float
    microlensing_relative_proper_motion_murel_alpha_ref___1masyr: float
    microlensing_relative_proper_motion_murel_delta_ref___1masyr: float
    microlensing_relative_proper_motion_murel_l_ref___1masyr: float
    microlensing_relative_proper_motion_murel_b_ref___1masyr: float
    microlensing_relative_proper_motion_murel_lambda_ref___1masyr: float
    microlensing_relative_proper_motion_murel_beta_ref___1masyr: float
    # Transverse Velocity
    microlensing_transverse_velocity_vt__km1s: float
    microlensing_gamma_ld: float
    microlensing_blending_flux_fs0: float
    microlensing_blending_flux_fs1: float
    microlensing_blending_flux_fs2: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        self.microlensing_impact_parameter_u0 = lightcurve_master['u0lens1'].values[0]
        self.microlensing_event_angle_alpha__deg = lightcurve_master['alpha'].values[0]
        self.microlensing_time_of_peak_t0__days = lightcurve_master['t0lens1'].values[0]

        # Caustic Region of INfluence
        self.microlensing_tcroin_days = lightcurve_master['tcroin'].values[0]
        self.microlensing_ucroin = lightcurve_master['ucroin'].values[0]
        self.microlensing_rcroin = lightcurve_master['rcroin'].values[0]

        # Heliocentric and reference frame tE (in Sun and Roman Frames)
        self.microlensing_einstein_crossing_time_tE_ref__days = lightcurve_master['tE_ref'].values[0]
        self.microlensing_einstein_crossing_time_tE_helio__days = lightcurve_master['tE_helio'].values[0]
        self.microlensing_einstein_radius_rE__AU = lightcurve_master['rE'].values[0]
        self.microlensing_einstein_angular_radius_thetaE__mas = lightcurve_master['thetaE'].values[0]
        self.microlensing_source_radius_in_einstein_radii_rho = lightcurve_master['rho'].values[0]

        # Parallax
        self.microlensing_parallax_effect_piE = lightcurve_master['piE'].values[0]
        self.microlensing_parallax_effect_piE_East = lightcurve_master['piEE'].values[0]
        self.microlensing_parallax_effect_piE_North = lightcurve_master['piEN'].values[0]
        self.microlensing_parallax_effect_piE_par = lightcurve_master['piEll'].values[0]
        self.microlensing_parallax_effect_piE_rp = lightcurve_master['piErp'].values[0]

        # Heliocentric Relative Proper Motion
        self.microlensing_relative_proper_motion_murel_helio__1masyr = lightcurve_master['murel_helio'].values[0]
        self.microlensing_relative_proper_motion_murel_alpha_helio___1masyr = \
            lightcurve_master['murel_helio_alpha'].values[0]
        self.microlensing_relative_proper_motion_murel_delta_helio___1masyr = \
            lightcurve_master['murel_helio_delta'].values[0]
        self.microlensing_relative_proper_motion_murel_l_helio___1masyr = lightcurve_master['murel_helio_l'].values[0]
        self.microlensing_relative_proper_motion_murel_b_helio___1masyr = lightcurve_master['murel_helio_b'].values[0]
        self.microlensing_relative_proper_motion_murel_lambda_helio___1masyr = \
            lightcurve_master['murel_helio_lambda'].values[0]

        # Reference Frame Relative Proper Motion (Roman)
        self.microlensing_relative_proper_motion_murel_ref__1masyr = lightcurve_master['murel_ref'].values[0]
        self.microlensing_relative_proper_motion_murel_beta_ref___1masyr = lightcurve_master['murel_ref_beta'].values[0]
        self.microlensing_relative_proper_motion_murel_ref__1masyr = lightcurve_master['murel_ref'].values[0]
        self.microlensing_relative_proper_motion_murel_alpha_ref___1masyr = lightcurve_master['murel_ref_alpha'].values[
            0]
        self.microlensing_relative_proper_motion_murel_delta_ref___1masyr = lightcurve_master['murel_ref_delta'].values[
            0]
        self.microlensing_relative_proper_motion_murel_l_ref___1masyr = lightcurve_master['murel_ref_l'].values[0]
        self.microlensing_relative_proper_motion_murel_b_ref___1masyr = lightcurve_master['murel_ref_b'].values[0]
        self.microlensing_relative_proper_motion_murel_lambda_ref___1masyr = \
            lightcurve_master['murel_ref_lambda'].values[0]
        self.microlensing_relative_proper_motion_murel_beta_ref___1masyr = lightcurve_master['murel_ref_beta'].values[0]

        # Transverse Velocity
        self.microlensing_transverse_velocity_vt__km1s = lightcurve_master['vt'].values[0]
        '''To Do: add vtilde, V_N, V_E. I don't understand the difference between them right now. '''
        self.microlensing_gamma_ld = 0.36  # since its 0 in the csv ...
        self.microlensing_blending_flux_fs0 = lightcurve_master['Obs_0_fs'].values[0]
        self.microlensing_blending_flux_fs1 = lightcurve_master['Obs_1_fs'].values[0]
        self.microlensing_blending_flux_fs2 = lightcurve_master['Obs_2_fs'].values[0]


@dataclass
class WeightAndFlags:
    """
    Weight and Flags: Includes the raw weight (raww), final weight (w), and flags indicating
    certain conditions of the simulation (e.g., FSflag, flatsatFlag).
    Analysis Flags and Chi-square Values: Indicators for the analysis status, including
    flat lightcurve flag (flatchi2), chi-square values for different fits (chi2_0, chi2_1, chi2),
    and the normalization weight (normw).
    """
    weight_and_flags_u0max: float
    weight_and_flags_raw_weight: float
    weight_and_flags_final_weight: float
    weight_and_flags_Error_flag: int
    weight_and_flags_ObsGroup0_FS_flag: int
    weight_and_flags_ObsGroup1_FS_flag: int
    weight_and_flags_ObsGroup2_FS_flag: int
    weight_and_flags_ObsGroup0_flat_lightcurve_flag: int
    weight_and_flags_ObsGroup1_flat_lightcurve_flag: int
    weight_and_flags_ObsGroup2_flat_lightcurve_flag: int
    weight_and_flags_ObsGroup0_flat_chi2: float
    weight_and_flags_ObsGroup1_flat_chi2: float
    weight_and_flags_ObsGroup2_flat_chi2: float
    weight_and_flags_ObsGroup0_chi2: float
    weight_and_flags_ObsGroup1_chi2: float
    weight_and_flags_ObsGroup2_chi2: float
    weight_and_flags_LC_output_flag: int

    def __init__(self, lightcurve_master: pd.DataFrame):
        self.weight_and_flags_u0max = lightcurve_master['u0max'].values[0]
        self.weight_and_flags_raw_weight = lightcurve_master['raw_weight'].values[0]
        self.weight_and_flags_final_weight = lightcurve_master['weight'].values[0]

        self.weight_and_flags_Error_flag = lightcurve_master['ErrorFlag'].values[0]

        self.weight_and_flags_ObsGroup0_FS_flag = lightcurve_master['ObsGroup_0_FiniteSourceflag'].values[0]
        self.weight_and_flags_ObsGroup1_FS_flag = lightcurve_master['ObsGroup_1_FiniteSourceflag'].values[0]
        self.weight_and_flags_ObsGroup2_FS_flag = lightcurve_master['ObsGroup_2_FiniteSourceflag'].values[0]

        self.weight_and_flags_ObsGroup0_flat_lightcurve_flag = lightcurve_master['ObsGroup_0_flatlc'].values[0]
        self.weight_and_flags_ObsGroup1_flat_lightcurve_flag = lightcurve_master['ObsGroup_1_flatlc'].values[0]
        self.weight_and_flags_ObsGroup2_flat_lightcurve_flag = lightcurve_master['ObsGroup_2_flatlc'].values[0]

        self.weight_and_flags_ObsGroup0_flat_chi2 = lightcurve_master['ObsGroup_0_flatchi2'].values[0]
        self.weight_and_flags_ObsGroup1_flat_chi2 = lightcurve_master['ObsGroup_1_flatchi2'].values[0]
        self.weight_and_flags_ObsGroup2_flat_chi2 = lightcurve_master['ObsGroup_2_flatchi2'].values[0]

        self.weight_and_flags_ObsGroup0_chi2 = lightcurve_master['ObsGroup_0_chi2'].values[0]
        self.weight_and_flags_ObsGroup1_chi2 = lightcurve_master['ObsGroup_1_chi2'].values[0]
        self.weight_and_flags_ObsGroup2_chi2 = lightcurve_master['ObsGroup_2_chi2'].values[0]
        self.weight_and_flags_LC_output_flag = lightcurve_master['LCOutput'].values[0]


@dataclass
class PlanetProperties:
    """
    Planet Properties: For simulations involving planets,
    this section lists the planet's mass (Mp),
    semi-major axis (a), inclination (inc), orbital phase (phase), mass ratio to the host star (q),
    separation (s), and orbital period (period).
    """
    planet_mass_ratio: float
    planet_mass__msun: float
    lens_system_mass__msun: float
    lens_planet_mass_ver: float
    lens_host_mass__msun: float
    planet_semi_major_axis__au: float
    planet_separation: float
    planet_inclination__deg: float
    planet_orbital_phase: float
    planet_orbital_period__years: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        # q: mass ratio
        self.planet_mass_ratio = lightcurve_master['Planet_q'].values[0]
        # Mp: mass of the planet
        self.planet_mass__msun = lightcurve_master['Planet_mass'].values[0]
        # mass of the planet calculated from the mass ratio
        # Ml: mass of the lens
        self.lens_system_mass__msun = lightcurve_master['Lens_Mass'].values[0]

        self.lens_planet_mass_ver = (self.planet_mass_ratio / (
                self.planet_mass_ratio + 1)) * self.lens_system_mass__msun
        # mass of the host star
        self.lens_host_mass__msun = self.lens_system_mass__msun / (1 + self.planet_mass_ratio)

        # Distances
        # a: semi-major axis
        self.planet_semi_major_axis__au = lightcurve_master['Planet_semimajoraxis'].values[0]
        # separation
        self.planet_separation = lightcurve_master['Planet_s'].values[0]
        self.planet_inclination__deg = lightcurve_master['Planet_inclination'].values[0]
        self.planet_orbital_phase = lightcurve_master['Planet_orbphase'].values[0]
        # period
        self.planet_orbital_period__years = lightcurve_master['Planet_period'].values[0]



import numpy as np
from jasmine.constants import general_physics_constants as astroconstants


def absolute_pi_E_calculator(pi_E_N_, pi_E_E_,
                             pi_E_N_error_=None, pi_E_E_error_=None,
                             is_error_included_=False):
    abs_piE = np.sqrt(pi_E_N_ ** 2 + pi_E_E_ ** 2)
    if is_error_included_:
        abs_piE_error = np.sqrt(((pi_E_N_ * pi_E_N_error_) ** 2 + (pi_E_E_ * pi_E_E_error_) ** 2) / (abs_piE ** 2))
        return abs_piE, abs_piE_error
    else:
        return abs_piE


def lens_mass__solar_mass_calculator(theta_E_, pi_E_,
                                     theta_E_error_=None, pi_E_error_=None,
                                     is_error_included_=False):
    kappa = astroconstants.kappa_mas_over_solar_mass
    lens_mass = theta_E_ / (kappa * pi_E_)
    if is_error_included_:
        lens_mass_error = np.sqrt((theta_E_error_ ** 2 / ((kappa ** 2) * (pi_E_ ** 2))) +
                                  (((theta_E_ ** 2) * (pi_E_error_ ** 2)) / ((kappa ** 2) * (pi_E_ ** 4))))
        return lens_mass, lens_mass_error
    else:
        return lens_mass


def planet_mass__earth_mass_calculator(mass_ratio_, lens_mass_solar_mass_,
                                       mass_ratio_error_=None, lens_mass_solar_mass_error_=None,
                                       is_error_included_=False):
    planet_mass__solar_mass = (lens_mass_solar_mass_ * mass_ratio_) / (1 + mass_ratio_)
    planet_mass__earth_mass = planet_mass__solar_mass * astroconstants.solar_mass__earth_mass
    if is_error_included_:
        partial_ML = mass_ratio_ / (1 + mass_ratio_)
        partial_q = lens_mass_solar_mass_ / (1 + mass_ratio_) ** 2

        planet_mass_error = np.sqrt(
            (partial_ML * lens_mass_solar_mass_error_) ** 2 +
            (partial_q * mass_ratio_error_) ** 2
        )
        planet_mass_error__earth_mass = planet_mass_error * astroconstants.solar_mass__earth_mass
        return planet_mass__earth_mass, planet_mass_error__earth_mass

    else:
        return planet_mass__earth_mass



import os
import pandas as pd
import jasmine.classes_and_files_reader.gullsrges_lightcurve_cls as lc
from jasmine.constants.limb_darkening_parameters import *
from astropy.coordinates import SkyCoord
from astropy import units


def creating_all_directories(*,
                             folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events',
                             master_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data',
                             data_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data'):
    if os.path.isdir(folder_path_):
        print(f' Found directory {folder_path_}! Continuing.')
    else:
        print(f' Error: {folder_path_} not found.')
    ##########
    # get lc numbers from data master files created by the data_challenge prep module
    ##########
    # pass dataframe index, subrun, event_id to CreatingDirectories/Files
    light_curve_master = pd.read_csv(f'{master_input_folder_path_}/OMPLLD_croin_cassan_0.sample.csv')
    sample_index = light_curve_master.index
    subrun = light_curve_master['SubRun']
    event_id = light_curve_master['EventID']
    for index in range(sample_index.shape[0]):
        creating_directories(sample_index=sample_index[index],subrun=subrun[index],event_id=event_id[index],
                             folder_path_=folder_path_)
        creating_files(sample_index=sample_index[index],subrun=subrun[index],event_id=event_id[index],
                       output_folder_path_=folder_path_,
                       master_input_folder_path_=master_input_folder_path_,
                       data_input_folder_path_=data_input_folder_path_)

    return None


def creating_directories(*, sample_index,subrun,event_id,
                         folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events'):
    """
    Create directories for the gulls event following RTModel structure
    """
    event_folder_ = f'{folder_path_}/event_{sample_index:04}_{subrun:03}_{event_id:04}'
    data_folder_ = f'{event_folder_}/Data'
    if os.path.isdir(event_folder_):
        raise FileExistsError(f'Directory {event_folder_} already exists!')
    else:
        os.mkdir(event_folder_)
        print(f'Folder {event_folder_} created!')
    if os.path.isdir(data_folder_):
        raise FileExistsError(f'Directory {data_folder_} already exists!')
    else:
        os.mkdir(data_folder_)
        print(f'Folder {data_folder_} created!')


def creating_files(*, sample_index,subrun,event_id,
                   output_folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events',
                   master_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data',
                   data_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data'):
    """
    Create files for the data challenge event following RTModel structure
    """
    data_folder_ = f'{output_folder_path_}/event_{sample_index:04}_{subrun:03}_{event_id:04}/Data'
    filters = ['W146', 'Z087', 'K213']
    ld_par = [W149, Z087, K213] # Order must match order in FilterToData.txt
    the_lightcurve = lc.LightcurveEventGULLSRGES(sample_index, data_folder=master_input_folder_path_)
    lightcurve_data_df_list = the_lightcurve.lightcurve_data()
    for i in range(3):
        filter_ = filters[i]
        if os.path.exists(f'{data_folder_}/Roman{filter_}sat{i}.dat'):
            raise FileExistsError(f'File Roman{filter_}sat{i}.dat already exists in directory {data_folder_}')
        else:
            with open(f'{data_folder_}/Roman{filter_}sat{i}.dat', 'w') as f:
                f.write('# Mag err HJD-2450000\n')
            lightcurve_data_df_list[i].to_csv(f'{data_folder_}/Roman{filter_}sat{i}.dat', sep=' ',
                                          index=False, header=False, mode='a')
            print(f'File Roman{filter_}sat{i}.dat created in {data_folder_}.')
        i += 1
    # write LimbDarkening.txt file    
    with open(data_folder_+'/LimbDarkening.txt','w') as f:
        for mu in ld_par:
            f.write(f'{mu}\n')



    # Create the file with the event information
    event_ra = the_lightcurve.event_info.event_right_ascension__deg
    event_dec = the_lightcurve.event_info.event_declination__deg
    event_coordinates = SkyCoord(event_ra, event_dec, unit=(units.deg, units.deg), obstime='J2000')
    rtmodel_event_coordinates = (event_coordinates.to_string('hmsdms').replace('h', ':').replace('m', ':')
                                 .replace('d', ':').replace('s', ''))
    if os.path.exists(f'{data_folder_}/event.coordinates'):
        raise FileExistsError(f'File event.coordinates already exists in directory {data_folder_}')
    else:
        with open(f'{data_folder_}/event.coordinates', 'w') as f:
            f.write(f'{rtmodel_event_coordinates}')
        print(f'File event.coordinates created in {data_folder_}.')


if __name__ == '__main__':
    folder_to_be_created = '/Users/jmbrashe/VBBOrbital/GULLS/events'
    input_folder = '/Volumes/JonDrive/GULLS/gulls_orbital_motion/repsample'
    satellite_folder = '/Users/jmbrashe/VBBOrbital/GULLS/satellitedir'
    creating_all_directories(folder_path_=folder_to_be_created,
                             master_input_folder_path_=input_folder, data_input_folder_path_=input_folder)
    #ephtools.creating_ephemerides(satellite_folder_path_=satellite_folder, ephemerides_path_=input_folder)



"""
You must run rtmodel_wrapper_for_roman_simulations.py first to generate the input CSV file.
"""
import pandas as pd
import numpy as np


def chi2_cut(single_lens_chi2, binary_lens_chi2, threshold=500):
    """
    Check if the difference between the single lens model chi2 and the best binary lens model chi2 exceeds the threshold.
    """
    return (single_lens_chi2 - binary_lens_chi2) > threshold


def acceptable_mass_ratio_range(model_mass_ratio, true_mass_ratio, log10_threshold=0.301):
    """
    Check if the absolute log10 difference between the model mass ratio and the true mass ratio is within the threshold 0.301
    10^0.301 ~ 2 . So model_q should not exceed 2*true_q, and vice versa.
    """
    ratio = model_mass_ratio / true_mass_ratio
    return np.abs(np.log10(ratio)) <= log10_threshold


def best_model_generator(dataframe):
    # best chi2 of Ls and LX (Exclude LO cause crazy orbits for now)
    dataframe['best_chi2'] = dataframe[['LS_chi2', 'LX_chi2']].min(axis=1)
    # name of the lowest chi2 models:
    dataframe['best_chi2_name'] = dataframe[['LS_chi2', 'LX_chi2']].idxmin(axis=1)
    # save the best mass ratio according to the lowest chi2 model:
    dataframe['best_mass_ratio'] = dataframe.apply(
        lambda row: row[f"{str(row['best_chi2_name']).replace('_chi2', '')}_mass_ratio"],
        axis=1)
    return dataframe


def main(file_path_, run_version_, root_path_, chi2_thresholds=None, saving_new_df=False):
    if chi2_thresholds is None:
        chi2_thresholds = [None, 0, 100, 200, 300, 400, 500, 600, 900, 1000]
    df = pd.read_csv(file_path_)
    complete_df = best_model_generator(df)
    print('Total number of events in the original DataFrame:', len(complete_df))
    for chi2_threshold in chi2_thresholds:
        if chi2_threshold is None:
            passed_chi2_df = complete_df.copy()
        else:
            complete_df[f'chi2_cut_{chi2_threshold}'] = chi2_cut(complete_df['PS_chi2'], complete_df['best_chi2'],
                                                                 threshold=chi2_threshold)
            # NEW DF for only those that passed the chi2 cut
            passed_chi2_df = complete_df[complete_df[f'chi2_cut_{chi2_threshold}']].copy()
            print('Number of events that had a best_PS_fit chi2 - best_binary_lens_fit chi2 > ', chi2_threshold, ' :',
                  len(passed_chi2_df))
            print('Number of events that did NOT pass the chi2 cut', chi2_threshold, ' :',
                  len(complete_df[~complete_df[f'chi2_cut_{chi2_threshold}']].copy()))
        # Now apply mass ratio cut for those that passed chi2 cut
        passed_chi2_df[f'acceptable_mass_ratio_range_chi2_{chi2_threshold}'] = acceptable_mass_ratio_range(
            passed_chi2_df['best_mass_ratio'],
            passed_chi2_df['true_mass_ratio'],
            log10_threshold=0.301
        )
        print('Of them, number of events within the mass ratio acceptable range:',
              len(passed_chi2_df[passed_chi2_df[f'acceptable_mass_ratio_range_chi2_{chi2_threshold}']]),
              f", {100 * len(passed_chi2_df[passed_chi2_df[f'acceptable_mass_ratio_range_chi2_{chi2_threshold}']]) / len(passed_chi2_df):.2f}%")
        print('Of them, number of events that exceed the mass ratio ratio cut / not acceptable:',
              len(passed_chi2_df[~passed_chi2_df[f'acceptable_mass_ratio_range_chi2_{chi2_threshold}']]),
              f", {100 * len(passed_chi2_df[~passed_chi2_df[f'acceptable_mass_ratio_range_chi2_{chi2_threshold}']]) / len(passed_chi2_df):.2f}%")
        print()
        if saving_new_df:
            new_df = passed_chi2_df[['event_name', 'true_mass_ratio', 'LS_mass_ratio', 'LX_mass_ratio',
                                     'PS_chi2', 'PX_chi2', 'LS_chi2', 'LX_chi2']].copy()

            output_csv = f"{root_path_}/{run_version_}_chi2_threshold_{chi2_threshold}.csv"
            new_df.to_csv(output_csv, index=False)
            print(f"Saved filtered DataFrame to {output_csv}")
            output_csv_2 = f"{root_path_}/{run_version_}_chi2_threshold_{chi2_threshold}_acceptable_mass_ratio.csv"
            acceptable_q_df = new_df[passed_chi2_df[f'acceptable_mass_ratio_range_chi2_{chi2_threshold}']]
            acceptable_q_df.to_csv(output_csv_2, index=False)
            print(f"Saved filtered DataFrame to {output_csv_2}")
            output_csv_3 = f"{root_path_}/{run_version_}_chi2_threshold_{chi2_threshold}_not_acceptable_mass_ratio.csv"
            not_acceptable_q_df = new_df[~passed_chi2_df[f'acceptable_mass_ratio_range_chi2_{chi2_threshold}']]
            not_acceptable_q_df.to_csv(output_csv_3, index=False)
            print(f"Saved filtered DataFrame to {output_csv_3}")


if __name__ == "__main__":
    # Load the CSV file generated by `rtmodel_wrapper_for_roman_simulations.py`
    run_version = 'v31'
    print(run_version, '\n')
    # ADAPT THIS
    if run_version == 'v31':
        root_path = '/Users/stela/Documents/Scripts/orbital_task/RTModel_runs/sample_rtmodel_v3.1'
        file_path = f'{root_path}/v3.1_true_and_rtmodel_fits.csv'
    elif run_version == 'v24':
        root_path = '/Users/stela/Documents/Scripts/orbital_task/RTModel_runs/sample_rtmodel_v2.4'
        file_path = f'{root_path}/true_and_rtmodel_fits.csv'
    elif run_version == 'ICGS':
        root_path = '/Users/stela/Documents/Scripts/orbital_task/RTModel_runs/154_failures_v24_v31'
        file_path = f'{root_path}/ICGS_true_and_rtmodel_fits_done.csv'
    else:
        raise ValueError("Invalid run_version. Use 'v31' or 'v24' or 'ICGS'.")
    main(file_path, run_version, root_path, saving_new_df=True)



import matplotlib.pyplot as plt
import pandas as pd


sample_df = pd.read_csv(
    '/Users/stela/Documents/Scripts/orbital_task/data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan.sample.csv')
sample_df['event_name'] = 'event_' + sample_df['SubRun'].astype(str) + '_' + sample_df['Field'].astype(str) + '_' + \
                          sample_df['EventID'].astype(str)

fig, ax = plt.subplots(figsize=(10*0.6, 6*0.6))

# Scatter plot
ax.scatter(sample_df['Planet_s'], sample_df['Planet_q'], alpha=0.7, edgecolor='k',
           s=10, label=f"Sample: # {len(sample_df['Planet_s'])}")

# Add labels
ax.set_ylabel('q: Mass Ratio', fontsize=12)
ax.set_xlabel('s: Separation', fontsize=12)
ax.set_title('Representative Sample', fontsize=14)
ax.set_xscale('log')
ax.set_yscale('log')
# plt.legend(loc='lower left')
plt.legend(loc='center left', bbox_to_anchor=(1, 0.5))

plt.tight_layout()
plt.savefig('representative_sample.png', dpi=300)

plt.close(fig)

fig, ax = plt.subplots(figsize=(10.4*0.6, 6*0.6))

sample_df_chi2_filtered = sample_df[sample_df['ObsGroup_0_chi2'] > 500].copy()
# Scatter plot
ax.scatter(sample_df['Planet_s'], sample_df['Planet_q'], alpha=0.7, edgecolor='k',
           s=10, label=f"Sample: # {len(sample_df['Planet_s'])}")
ax.scatter(sample_df_chi2_filtered['Planet_s'], sample_df_chi2_filtered['Planet_q'], alpha=0.7, edgecolor='k',
           s=10, label=fr"$\Delta \chi^2 > 500$: # {len(sample_df_chi2_filtered['Planet_s'])}")

# Add labels
ax.set_ylabel('q: Mass Ratio', fontsize=12)
ax.set_xlabel('s: Separation', fontsize=12)
ax.set_title('Representative Sample', fontsize=14)
ax.set_xscale('log')
ax.set_yscale('log')
# plt.legend(loc='lower left')
plt.legend(loc='center left', bbox_to_anchor=(1, 0.5))

plt.tight_layout()
plt.savefig('representative_sample_chi2_filtered.png', dpi=300)






import pandas as pd
import numpy as np
from dataclasses import dataclass
import VBMicrolensing
from moana import lens as mlens


@dataclass
class RTModelTemplateForBinaryLightCurve:
    """
    Generate a light curve based on a line of RTModel template
    """
    template_line: int
    separation_s: float
    mass_ratio_q: float
    impact_parameter_u0: float
    angle_alpha: float
    source_radius_rho: float
    pre_calculated_peak_tp1: float
    pre_calculated_peak_tp2: float
    einstein_time_tE: float
    peak_time_t0: float
    input_peak_t1: float
    input_peak_t2: float

    def __init__(self, *, template_line, path_to_template,
                 input_peak_t1, input_peak_t2):
        self.template_line = template_line
        self.path_to_template = path_to_template
        line_information = read_template(path_to_template).loc[template_line - 2]
        self.separation_s = line_information['s']
        self.mass_ratio_q = line_information['q']
        self.impact_parameter_u0 = line_information['u0']
        self.angle_alpha = line_information['alpha']
        self.source_radius_rho = line_information['rho']
        self.pre_calculated_peak_tp1 = line_information['tp1']
        self.pre_calculated_peak_tp2 = line_information['tp2']
        self.input_peak_t1 = input_peak_t1
        self.input_peak_t2 = input_peak_t2
        self.einstein_time_tE = tE_definer(t1=self.input_peak_t1,
                                           t2=self.input_peak_t2,
                                           tp1=self.pre_calculated_peak_tp1,
                                           tp2=self.pre_calculated_peak_tp2)
        self.peak_time_t0 = t0_definer(t1=self.input_peak_t1,
                                       tE=self.einstein_time_tE,
                                       tp1=self.pre_calculated_peak_tp1)
        self.results = None
        self.source_trajectory_y1 = None
        self.source_trajectory_y2 = None
        self.magnification = None
        self.times = None
        self.critical_curves = None
        self.caustics = None
        self.type_of_caustics = None

    def rtmodel_calculation_using_vbm(self, *, time_interval=None,
                                      parallax=False,
                                      orbital_motion=False,
                                      n_points=10000):
        """
        Calculate the light curve using the VBMicrolensing package
        :param time_interval:
        :param parallax:
        :param orbital_motion:
        :param n_points:
        :return:
        """
        vbm = VBMicrolensing.VBMicrolensing()

        separation_s = self.separation_s
        mass_ratio_q = self.mass_ratio_q
        alpha = self.angle_alpha
        impact_parameter_u0 = self.impact_parameter_u0
        einstein_time_tE = self.einstein_time_tE
        peak_time_t0 = self.peak_time_t0
        rho = self.source_radius_rho
        if time_interval is None:
            time_interval = np.linspace(peak_time_t0 - 2 * einstein_time_tE,
                                        peak_time_t0 + 2 * einstein_time_tE,
                                        n_points)
        if parallax:
            print('Edit here to a collection of parameters with parallax')
        if orbital_motion:
            print('Edit here to a collection of parameters with orbital motion')
        pr = [np.log(separation_s), np.log(mass_ratio_q), impact_parameter_u0,
              alpha, np.log(rho), np.log(einstein_time_tE), peak_time_t0]
        self.results = vbm.BinaryLightCurve(pr, time_interval)
        self.times = time_interval
        return self.results

    def rtmodel_magnification_using_vbm(self, *,
                                        time_interval=None,
                                        parallax=False,
                                        orbital_motion=False,
                                        n_points=10000):
        """
        Calculate the magnification using the VBMicrolensing package
        :param time_interval:
        :param parallax:
        :param orbital_motion:
        :param n_points:
        :return:
        """
        self.results = self.rtmodel_calculation_using_vbm(time_interval=time_interval,
                                                          parallax=parallax,
                                                          orbital_motion=orbital_motion,
                                                          n_points=n_points)

        self.magnification = self.results[0]
        return self.magnification, self.times

    def rtmodel_source_trajectory_using_vbm(self, *,
                                            time_interval=None,
                                            parallax=False,
                                            orbital_motion=False,
                                            n_points=10000):
        """
        Calculate the source trajectory using the VBMicrolensing package
        :param time_interval:
        :param parallax:
        :param orbital_motion:
        :param n_points:
        :return:
        """
        if self.results is None:
            self.results = self.rtmodel_calculation_using_vbm(time_interval=time_interval,
                                                              parallax=parallax,
                                                              orbital_motion=orbital_motion,
                                                              n_points=n_points)
        self.source_trajectory_y1 = self.results[1]
        self.source_trajectory_y2 = self.results[2]
        return self.source_trajectory_y1, self.source_trajectory_y2

    def rtmodel_caustics_using_vbm(self):
        """
        Calculate the caustics using the VBMicrolensing package
        :return:
        """
        vbm = VBMicrolensing.VBMicrolensing()
        caustics = vbm.Caustics(self.separation_s, self.mass_ratio_q)
        self.caustics = caustics
        return caustics

    def which_topology_from_moana(self):
        """
        Check the topology type of the binary lens system
        :return:
        """
        if self.separation_s < mlens.close_limit_2l(self.mass_ratio_q):
            self.type_of_caustics = 'close'
        elif self.separation_s <= mlens.wide_limit_2l(self.mass_ratio_q):
            self.type_of_caustics = 'ressonant'
        else:
            self.type_of_caustics = 'wide'
        return self.type_of_caustics

def tE_definer(*, t1, t2, tp1, tp2):
    """
    Equation 8 in Bozza 2024 https://arxiv.org/pdf/2405.04092
    :param t1: position of peak 1 in units of Einstein time
    :param t2: position of peak 2 in units of Einstein time
    :param tp1: pre-calculated position of peak 1 in units of Einstein time
    :param tp2: pre-calculated position of peak 2 in units of Einstein time
    :return:
    """
    tE = (t2 - t1) / (tp2 - tp1)
    return tE


def t0_definer(*, t1, tE, tp1):
    """
    Equation 9 in Bozza 2024 https://arxiv.org/pdf/2405.04092
    :param t1: position of peak 1 in units of Einstein time tE
    :param tE: Einstein time for that event, referred to function above
    :param tp1: pre-calculated position of peak 1 in units of Einstein time
    :return:
    """
    t0 = t1 - tE * tp1
    return t0


def read_template(path_to_template):
    template_df = pd.read_csv(path_to_template,
                              skiprows=1,
                              sep=r'\s+',
                              names=['s', 'q', 'u0', 'alpha', 'rho', 'tp1', 'tp2'],
                              )
    return template_df



from astropy.coordinates import SkyCoord
import astropy.units as u

# Function to convert RA from 'HH:MM:SS' to decimal degrees
def ra_hms_to_deg(ra_hms):
    h, m, s = [float(i) for i in ra_hms.split(':')]
    ra_deg = 15 * (h + m / 60 + s / 3600)
    return ra_deg


# Function to convert Dec from 'DD:MM:SS' to decimal degrees
def dec_dms_to_deg(dec_dms):
    sign = -1 if dec_dms[0] == '-' else 1
    d, m, s = [float(i) for i in dec_dms.split(':')]
    dec_deg = sign * (abs(d) + m / 60 + s / 3600)
    return dec_deg


def convert_to_decimal_degrees(ra_hms, dec_dms):
    coord = SkyCoord(ra=ra_hms, dec=dec_dms, unit=(u.hourangle, u.deg))
    return coord.ra.deg, coord.dec.deg

def convert_degree_to_hms_dms(ra_deg, dec_deg):
    coord = SkyCoord(ra=ra_deg * u.deg, dec=dec_deg * u.deg)
    ra_hms = coord.ra.to_string(unit=u.hourangle, sep=':', precision=2)
    dec_dms = coord.dec.to_string(unit=u.deg, sep=':', precision=2, alwayssign=True)
    return ra_hms, dec_dms


if __name__ == '__main__':
    # Answer for the example below
    # RA,Dec
    ra, dec = '17:46:48.87','-33:42:06.9'
    # RA_deg,Dec_deg
    ra_deg, dec_deg = 266.70362499999993,-33.70191666666667

    print('HMS DMS TO DECIMAL DEGREE')
    print(f'TRUE VALUES ra_deg: {ra_deg}, dec_deg: {dec_deg}')
    print(convert_to_decimal_degrees(ra, dec))

    print('DECIMAL DEGREE TO HMS DMS')
    print(f'TRUE VALUES ra: {ra}, dec: {dec}')
    print(convert_degree_to_hms_dms(ra_deg, dec_deg))



import numpy as np
from jasmine.investigator.astrophysics_formulas import absolute_pi_E_calculator

def relative_difference_from_true_value_percentage_calculator(true_value, approximated_value):
    percent_error = 100 * np.absolute((true_value - approximated_value) / true_value)
    return percent_error


def relative_difference_from_true_value_calculator(true_value, approximated_value):
    relative_difference = np.absolute((true_value - approximated_value) / true_value)
    return relative_difference

def standardized_residual_calculator(true_value, predicted_value, predicted_errorbar):
    standardized_residual = (true_value - predicted_value) / predicted_errorbar
    return standardized_residual

def absolute_standardized_residual_calculator(true_value, predicted_value, predicted_errorbar):
    absolute_standardized_residual = np.absolute(standardized_residual_calculator(true_value, predicted_value, predicted_errorbar))
    return absolute_standardized_residual

def predicted_value_predicted_errorbar_ratio_calculator(predicted_value, predicted_errorbar):
    ratio = predicted_value / predicted_errorbar
    return ratio

def creating_statistics_table(working_data_, parallax_24_summary_df_, *, is_save_=False,
                              output_path_='/Users/stela/Documents/Scripts/orbital_task/RTModel_plots'):
    """
    Create a table with the statistics of the parallax
    :return:
    """
    working_data_ = working_data_.set_index('event_name')
    for model_type in ['LX', 'LO']:
        # copying RTModel results for parallax
        # LX/LO

        # Absolute piE calculator:
        working_data_[f'{model_type}_model_piE'], working_data_[
            f'{model_type}_model_piE_error'] = absolute_pi_E_calculator(parallax_24_summary_df_[f'{model_type}_piEN'],
                                                                        parallax_24_summary_df_[f'{model_type}_piEE'],
                                                                        pi_E_N_error_=parallax_24_summary_df_[
                                                                            f'{model_type}_piEN_error'],
                                                                        pi_E_E_error_=parallax_24_summary_df_[
                                                                            f'{model_type}_piEE_error'],
                                                                        is_error_included_=True)
        # Saving to working dataframe piEE and piEN for the models LX/LO
        for piE_type in ['piEE', 'piEN']:
            working_data_[f'{model_type}_model_{piE_type}'] = parallax_24_summary_df_[f'{model_type}_{piE_type}']
            working_data_[f'{model_type}_model_{piE_type}_error'] = parallax_24_summary_df_[
                f'{model_type}_{piE_type}_error']

        # Calculating parallax residuals
        for piE_type in ['piE', 'piEE', 'piEN']:
            # Calculating parallax piE, piEE, piEN standardized residuals
            working_data_[f'{model_type}_{piE_type}_standardized_residual'] = standardized_residual_calculator(
                working_data_[f'{piE_type}'],
                working_data_[f'{model_type}_model_{piE_type}'],
                working_data_[f'{model_type}_model_{piE_type}_error'])

            # Calculating parallax piE, piEE, piEN absolute standardized residual
            working_data_[
                f'{model_type}_{piE_type}_absolute_standardized_residual'] = absolute_standardized_residual_calculator(
                working_data_[f'{piE_type}'], working_data_[f'{model_type}_model_{piE_type}'],
                working_data_[f'{model_type}_model_{piE_type}_error'])

            # Ratio of predicted value to predicted error
            working_data_[
                f'{model_type}_{piE_type}_predicted_value_predicted_errorbar_ratio'] = predicted_value_predicted_errorbar_ratio_calculator(
                working_data_[f'{model_type}_model_{piE_type}'], working_data_[f'{model_type}_model_{piE_type}_error'])

            # Absolute Ratio of predicted value to predicted error
            working_data_[f'{model_type}_{piE_type}_absolute_predicted_value_predicted_errorbar_ratio'] = np.abs(
                working_data_[f'{model_type}_{piE_type}_predicted_value_predicted_errorbar_ratio'])

            # Inverse. Ratio of predicted error to predicted value
            working_data_[f'{model_type}_{piE_type}_predicted_errorbar_predicted_value_ratio'] = working_data_[
                                                                                                 f'{model_type}_model_{piE_type}_error'] / \
                                                                                              working_data_[
                                                                                                 f'{model_type}_model_{piE_type}']

            # Inverse. Absolute Ratio of predicted error to predicted value
            working_data_[f'{model_type}_{piE_type}_absolute_predicted_errorbar_predicted_value_ratio'] = np.abs(
                working_data_[f'{model_type}_{piE_type}_predicted_errorbar_predicted_value_ratio'])

            # Calculating parallax piE, piEE, piEN relative error
            working_data_[f'{model_type}_{piE_type}_relative_difference_from_true_value'] = relative_difference_from_true_value_calculator(
                working_data_[f'{piE_type}'],
                working_data_[f'{model_type}_model_{piE_type}'])
    if is_save_:
        working_data_.to_csv(f'{output_path_}.csv')
    return working_data_


if __name__ == '__main__':
    # Example usage

    # LOAD YOUR DATA as a pandas DataFrame
    import pandas as pd

    # Load the sample master file
    representative_sample_df = pd.read_csv(
        '/Users/stela/Documents/Scripts/orbital_task/data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan.sample.csv')
    representative_sample_df['event_name'] = 'event_' + representative_sample_df['SubRun'].astype(str) + '_' + \
                                             representative_sample_df['Field'].astype(str) + '_' + \
                                             representative_sample_df['EventID'].astype(str)
    # LOAD your parallax results
    parallax_24_summary_df =  pd.read_csv(f'/RTModel_runs/sample_rtmodel_v2.4/all_sample_rtmodel_v2.4_parallax.csv',
                                          index_col=['event_name'])

    # WORKING DATA
    # Preparing columns of interest
    columns_to_copy = ['event_name', 'piE', 'piEE', 'piEN', 'final_weight']
    working_data = representative_sample_df[columns_to_copy].copy()

    # IF SAVING SET AN OUTPUT
    output_for_table = ('/Users/stela/Documents/Scripts/orbital_task/RTModel_plots/'
                        'Notebook_statiscs_by_cuts/rtmodelv2.4_statistics')
    working_data = creating_statistics_table(working_data, parallax_24_summary_df,
                                             is_save_=True, output_path_=output_for_table)


"""
This code creates the directories and files for the GULLS events launched in Nov2024 in the RTModel structure.
You also need to use
"""
import os
import pandas as pd
import jasmine.classes_and_files_reader.new_gullsrges_lightcurve_cls as lc
from jasmine.constants.limb_darkening_parameters import *
from astropy.coordinates import SkyCoord
from astropy import units


def getting_all_subruns_and_fields_folder_name(data_folder_path_):
    """
    Creates a .csv file to control the subruns and fields pairs
    :param data_folder_path_:
    :return:
    """
    subrun_field_pairs = []
    for directory in os.listdir(data_folder_path_):
        if directory.startswith("OMPLDG_croin_cassan_") and directory != "OMPLDG_croin_cassan_sample":
            parts = directory.split("_")
            subrun = parts[-3]
            field = parts[-2]
            subrun_field_pairs.append({"SubRun": subrun, "Field": field})
            print(f'SubRun is: {subrun} | Field is: {field}')

    df = pd.DataFrame(subrun_field_pairs)
    output_csv = f'{data_folder_path_}/subrun_field_pairs.csv'
    df.to_csv(output_csv, index=False)
    print(f"CSV file saved as {output_csv}")


def does_this_lightcurve_exist(subrun_, field_, event_id_, data_folder_path_):
    directory = f"{data_folder_path_}/OMPLDG_croin_cassan_{subrun_}_{field_}_lc"
    lightcurve_path = f"{directory}/OMPLDG_croin_cassan_{subrun_}_{field_}_{event_id_}.det.lc"
    return os.path.exists(lightcurve_path)


def converting_mini_tables_in_one_general_csv(data_folder_path_):
    subrun_field_csv = f"{data_folder_path_}/subrun_field_pairs.csv"
    output_csv = f"{data_folder_path_}/10k_master.csv"

    subrun_field_df = pd.read_csv(subrun_field_csv)  # Loading all the SubRun-Field pairs
    combined_data = []  # df to be concatenated
    for _, row in subrun_field_df.iterrows():
        subrun = row["SubRun"]
        field = row["Field"]
        folder_name = f"OMPLDG_croin_cassan_{subrun}_{field}_lc"
        out_file_name = f"OMPLDG_croin_cassan_{subrun}_{field}.out"
        out_file_path = os.path.join(data_folder_path_, folder_name, out_file_name)
        if os.path.getsize(out_file_path) > 0:
            df = pd.read_csv(out_file_path, delim_whitespace=True)
            df['lcname'] = (f"OMPLDG_croin_cassan_{subrun}_{field}_lc/"
                            f"OMPLDG_croin_cassan_{subrun}_{field}_" + df['EventID'].astype(str) + ".det.lc")
            # EventID	SubRun	Field
            combined_data.append(df)  # Append the df to the combined list
        else:
            print(f"Skipped empty file: {out_file_path}")

    # Combine all dataframes into one
    combined_df = pd.concat(combined_data, ignore_index=True)
    df_filtrado = combined_df[combined_df.apply(lambda df_row: does_this_lightcurve_exist(df_row['SubRun'],
                                                                                          df_row['Field'],
                                                                                          df_row['EventID'],
                                                                                          data_folder_path_), axis=1)]
    df_filtrado.to_csv(output_csv, index=False)
    print(f"Master CSV file saved as {output_csv}")


def csv_file_without_sample(data_folder_path_):
    sample_df = pd.read_csv(f'{data_folder_path_}/OMPLDG_croin_cassan.sample.csv')
    master_df = pd.read_csv(f'{data_folder_path_}/20k_master.csv')
    columns_comparison = ['SubRun', 'Field', 'EventID']

    # Perform a merge with an indicator to identify differences
    merged = master_df.merge(sample_df, how='left', on=columns_comparison, indicator=True)

    # Filter rows that are only in the first DataFrame
    result_df = merged[merged['_merge'] == 'left_only'].drop(columns=['_merge'])

    # Save or inspect the result
    result_df.to_csv(f"{data_folder_path_}/20k_master_minus_sample.csv", index=False)
    print(f"Filtered DataFrame has {len(result_df)} rows.")


def creating_all_directories(*,
                             folder_to_be_created_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events',
                             master_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data',
                             data_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data'):
    if os.path.isdir(folder_to_be_created_path_):
        print(f' Found directory {folder_to_be_created_path_}! Continuing.')
    else:
        print(f' Error: {folder_to_be_created_path_} not found.')
    ##########
    # get lc numbers from data master files created by the data_challenge prep module
    ##########
    # pass dataframe index, subrun_, event_id_ to CreatingDirectories/Files
    light_curve_master_file = f'{master_input_folder_path_}/10k_master.csv'
    light_curve_master = pd.read_csv(light_curve_master_file)
    for _, row in light_curve_master.iterrows():
        subrun = row['SubRun']
        event_id = row['EventID']
        lc_filename = row['lcname']
        field = row['Field']
        creating_directories(subrun=subrun, event_id=event_id, field=field,
                             general_folder_path_=folder_to_be_created_path_)
        creating_files(subrun=subrun, event_id=event_id, field=field,
                       lightcurve_name=lc_filename,
                       general_output_folder_path_=folder_to_be_created_path_,
                       master_file_path_=light_curve_master_file,
                       data_input_folder_path_=data_input_folder_path_)
        print()

    return None


def creating_directories(*, subrun, event_id, field,
                         general_folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events'):
    """
    Create directories for the gulls event following RTModel structure
    """
    event_folder_ = f'{general_folder_path_}/event_{subrun}_{field}_{event_id}'
    data_folder_ = f'{event_folder_}/Data'
    if os.path.isdir(event_folder_):
        raise FileExistsError(f'Directory {event_folder_} already exists!')
    else:
        os.mkdir(event_folder_)
        print(f'Folder {event_folder_} created!')
    if os.path.isdir(data_folder_):
        raise FileExistsError(f'Directory {data_folder_} already exists!')
    else:
        os.mkdir(data_folder_)
        print(f'Folder {data_folder_} created!')


def creating_files(*, subrun, event_id, field, lightcurve_name,
                   general_output_folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events',
                   master_file_path_='/Users/sishitan/Documents/Scripts/jasmine/data',
                   data_input_folder_path_='/Users/sishitan/Documenxts/Scripts/jasmine/data'):
    """
    Create files for the data challenge event following RTModel structure
    """
    data_to_be_created_folder_ = f'{general_output_folder_path_}/event_{subrun}_{field}_{event_id}/Data'
    filters = ['W146', 'Z087', 'K213']
    ld_par = [W149, Z087, K213]  # Order must match order in FilterToData.txt
    the_lightcurve = lc.LightcurveEventGULLSRGES_NameBased(lightcurve_name=lightcurve_name,
                                                           data_folder=data_input_folder_path_,
                                                           master_file_path=master_file_path_)
    lightcurve_data_df_list = the_lightcurve.lightcurve_data()
    for i in range(1, 4):
        filter_ = filters[i - 1]
        if os.path.exists(f'{data_to_be_created_folder_}/Roman{filter_}sat{i}.dat'):
            raise FileExistsError(
                f'File Roman{filter_}sat{i}.dat already exists in directory {data_to_be_created_folder_}')
        else:
            with open(f'{data_to_be_created_folder_}/Roman{filter_}sat{i}.dat', 'w') as f:
                f.write('# Mag err HJD-2450000\n')
            lightcurve_data_df_list[i - 1].to_csv(f'{data_to_be_created_folder_}/Roman{filter_}sat{i}.dat', sep=' ',
                                                  index=False, header=False, mode='a')
            print(f'File Roman{filter_}sat{i}.dat created in {data_to_be_created_folder_}.')
        i += 1
    # write LimbDarkening.txt file
    with open(data_to_be_created_folder_ + '/LimbDarkening.txt', 'w') as f:
        for mu in ld_par:
            f.write(f'{mu}\n')

    # Create the file with the event information
    event_ra = the_lightcurve.event_info.event_right_ascension__deg
    event_dec = the_lightcurve.event_info.event_declination__deg
    event_coordinates = SkyCoord(event_ra, event_dec, unit=(units.deg, units.deg), obstime='J2000')
    rtmodel_event_coordinates = (event_coordinates.to_string('hmsdms').replace('h', ':').replace('m', ':')
                                 .replace('d', ':').replace('s', ''))
    if os.path.exists(f'{data_to_be_created_folder_}/event.coordinates'):
        raise FileExistsError(f'File event.coordinates already exists in directory {data_to_be_created_folder_}')
    else:
        with open(f'{data_to_be_created_folder_}/event.coordinates', 'w') as f:
            f.write(f'{rtmodel_event_coordinates}')
        print(f'File event.coordinates created in {data_to_be_created_folder_}.')


if __name__ == '__main__':
    # NEW JAN 8
    folder_path = "/Users/stela/Documents/Scripts/orbital_task/data/gulls_orbital_motion_extracted"
    # getting_all_subruns_and_fields_folder_name(folder_path)
    # converting_mini_tables_in_one_general_csv(folder_path)
    csv_file_without_sample(folder_path)
    # TODO I AM HERE
    # general_path_outputs = '/discover/nobackup/sishitan/orbital_task'
    # # general_path_outputs = '/Users/stela/Documents/Scripts/orbital_task'
    # folder_to_be_created = f'{general_path_outputs}/RTModel_runs'
    # master_input_folder = f'{general_path_outputs}/data/gulls_orbital_motion_extracted'
    # data_input_folder = f'{general_path_outputs}/data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan_sample'
    # creating_all_directories(folder_to_be_created_path_=folder_to_be_created,
    #                          master_input_folder_path_=master_input_folder,
    #                          data_input_folder_path_=data_input_folder)



"""
Called by RTModel_results_reader.py - check that
"""
import pandas as pd

# from jasmine.classes_and_files_reader.RTModel_results_reader import list_of_events_from_sample_df # commenting it off because there is a circular import
from jasmine.classes_and_files_reader.new_gullsrges_lightcurve_cls import LightcurveEventGULLSRGES_NameBased
from jasmine import ModelResults


def get_summary_of_q_s_chi2_per_event(folder_path_, are_there_binary_solutions_):
    """
    This function creates a summary of q and s for the top 1 models + trues values for the given event
    """
    general_path_ = folder_path_.split('/RTModel_runs')[0] + '/data/gulls_orbital_motion_extracted'
    master_file_path = f'{general_path_}/OMPLDG_croin_cassan.sample.csv'
    data_folder_path = f'{general_path_}/OMPLDG_croin_cassan_sample'
    event_identification = folder_path_.split('/')[-1].split('vent_')[-1]
    lightcurve_name = f'OMPLDG_croin_cassan/OMPLDG_croin_cassan_{event_identification}.det.lc'
    event_name = folder_path_.split('/')[-1]
    subrun = event_name.split('_')[1]
    field = event_name.split('_')[2]
    event_id = event_name.split('_')[3]

    the_lightcurve_event = LightcurveEventGULLSRGES_NameBased(lightcurve_name, data_folder_path, master_file_path)
    true_q = the_lightcurve_event.planet.planet_mass_ratio
    true_s = the_lightcurve_event.planet.planet_separation

    data_to_be_saved = {'event_name': [event_name,],
                        'subrun': [subrun,],
                        'field': [field,],
                        'event_id': [event_id,],
                        'true_q': [true_q, ],
                        'true_s': [true_s, ], }
    if are_there_binary_solutions_:
        top_1_of_each = pd.read_csv(folder_path_ + '/Models/chi2_top1_of_each_binary_lens_model.csv')
        for model_name in top_1_of_each['model']:
            model_type = model_name[0:2]
            model_path = folder_path_ + '/Models/' + model_name + '.txt'
            model_parameters = ModelResults(model_path).model_parameters
            data_to_be_saved[f'{model_type}_q'] = [model_parameters.mass_ratio, ]
            data_to_be_saved[f'{model_type}_q_err'] = [model_parameters.mass_ratio_error, ]
            data_to_be_saved[f'{model_type}_s'] = [model_parameters.separation, ]
            data_to_be_saved[f'{model_type}_s_err'] = [model_parameters.separation_error, ]
            data_to_be_saved[f'{model_type}_chi2'] = [model_parameters.chi2, ]
    event_summary = pd.DataFrame(data_to_be_saved)
    event_summary.to_csv(folder_path_ + '/Models/event_summary_q_s.csv', index=False)
    return event_summary


def event_summary_q_s_wrapper(root_path_, list_of_events_, type_of_event_):
    """
    This function wraps up all the results for the given type of events, and produce a .csv with all of them
    """
    df_general = []
    for event_ in list_of_events_:
        df_for_one_event = pd.read_csv(f'{root_path_}/{event_}/Models/event_summary_q_s.csv')
        df_general.append(df_for_one_event)
    # Concatenate all the dataframes
    all_q_s_wrapper_df = pd.concat(df_general)
    all_q_s_wrapper_df.to_csv(f'{root_path_}/all_{type_of_event_}_q_s.csv', index=False)
    return all_q_s_wrapper_df


if __name__ == '__main__':
    # list_of_events = ['event_0_1000_1445',
    #                   'event_0_42_2848',
    #                   'event_0_762_407']
    # /discover/nobackup/sishitan/orbital_task/RTModel_runs/sample
    type_of_event = 'sample'
    runs_path = f'/discover/nobackup/sishitan/orbital_task/RTModel_runs/{type_of_event}'
    # list_of_events = list_of_events_from_sample_df(runs_path) # commenting it off because there is a circular import
    # general_path_outputs = '/Users/stela/Documents/Scripts/orbital_task/RTModel_runs/top10_piE'
    for event in list_of_events:
        folder_path = f'{runs_path}/{event}'
        get_summary_of_q_s_chi2_per_event(folder_path, True)
    event_summary_q_s_wrapper(runs_path, list_of_events, type_of_event)



"""
Wide-Field Instrument https://roman.gsfc.nasa.gov/science/WFI_technical.html
"""
from dataclasses import dataclass


@dataclass
class WFI_filter_specifications:
    filter_name: str
    min_micrometer: float
    max_micrometer: float
    center_micrometer: float
    width_micrometer: float
    corner_PSF_FWHM_arcsec: float
    corner_n_eff_pixel: float
    corner_peak_flux: float
    center_n_eff_pixel: float
    center_peak_flux: float

    def __init__(self, filter_name_):
        self.filter_name = filter_name_
        self.filter_properties = filter_properties_getter(self.filter_name)
        self.min_micrometer = self.filter_properties[0]
        self.max_micrometer = self.filter_properties[1]
        self.center_micrometer = self.filter_properties[2]
        self.width_micrometer = self.filter_properties[3]
        self.corner_PSF_FWHM_arcsec = self.filter_properties[4]
        self.corner_n_eff_pixel = self.filter_properties[5]
        self.corner_peak_flux = self.filter_properties[6]
        self.center_n_eff_pixel = self.filter_properties[7]
        self.center_peak_flux = self.filter_properties[8]


def filter_properties_getter(filter_name_):
    if filter_name_ == 'F062':
        filter_properties = [0.48, 0.76, 0.620, 0.280, 0.058, 7.35, 0.20341, 3.80, 0.49536]
    elif filter_name_ == 'F087':
        filter_properties = [0.76, 0.977, 0.869, 0.217, 0.073, 9.35, 0.16517, 4.04, 0.4838]
    elif filter_name_ == 'F106':
        filter_properties = [0.927, 1.192, 1.060, 0.265, 0.087, 10.96, 0.15060, 4.83, 0.44004]
    elif filter_name_ == 'F129':
        filter_properties = [1.131, 1.454, 1.293, 0.323, 0.106, 11.79, 0.14863, 6.63, 0.36874]
    elif filter_name_ == 'F158':
        filter_properties = [1.380, 1.774, 1.577, 0.394, 0.128, 12.63, 0.14343, 9.65, 0.29081]
    elif filter_name_ == 'F184':
        filter_properties = [1.683, 2.000, 1.842, 0.317, 0.146, 17.15, 0.11953, 15.52, 0.21361]
    elif filter_name_ == 'F213':
        filter_properties = [1.95, 2.30, 2.125, 0.35, 0.169, 20.38, 0.10831, 20.14, 0.17052]
    elif filter_name_ == 'F146':
        filter_properties = [0.927, 2.000, 1.464, 1.030, 0.105, 12.18, 0.14521, 7.37, 0.34546]
    else:
        raise ValueError(f'filter_name should be F062, F087, F106, F129, F158, F184, F213, F146')
    return filter_properties



kappa_mas_over_solar_mass = 8.144
solar_mass__kg = 1.988416e30
earth_mass__kg = 5.9722e24
jupiter_mass_kg = 1.898e27  # upate later
solar_mass__earth_mass = solar_mass__kg/earth_mass__kg
jupiter_mass__earth_mass = jupiter_mass_kg/earth_mass__kg
solar_mass__jupiter_mass = solar_mass__kg/jupiter_mass_kg



import pandas as pd
import numpy as np
import sys



sample_path = '/Users/jmbrashe/VBBOrbital/NEWGULLS/OMPLDG_croin_cassan.sample.csv'
df = pd.read_csv(sample_path)

event_name = sys.argv[1]
event_name = event_name[5:]
lc_name = f'OMPLDG_croin_cassan/OMPLDG_croin_cassan{event_name}.det.lc'
event = df[df['lcname']==lc_name].iloc[0,:]

phi = event['Planet_orbphase']*np.pi/180 #orbital phase
T = event['Planet_period']*365.25 # period
n = 2*np.pi/(T)
i = event['Planet_inclination']*np.pi/180
s = event['Planet_s']
a_au = event['Planet_semimajoraxis']
theta_e = event['thetaE']
DL = event['Lens_Dist']*206264806.24538 # convert to AU

a_rad = a_au/DL
a_mas = a_rad*(180/np.pi)*3600*1000
a = a_mas/theta_e

gamma = n*a/s
gamma1 = -(n/np.tan(phi))*((a/s)**2-1)
gamma2 = n*((a/s)**2)*np.cos(i)
gamma3 = -n*np.sqrt((a/s)**2-1)/np.tan(phi)

print(f'v_para, v_perp, v_radial (day^-1), period (days) = {gamma1},{gamma2},{gamma3},{T}')


from jasmine.files_organizer.ra_and_dec_conversions import convert_degree_to_hms_dms

def test_ra_conversion():
    # Answer for the example below
    # RA,Dec
    ra = '17:46:48.87'
    # RA_deg,Dec_deg
    ra_deg, dec_deg = 266.70362499999993,-33.70191666666667

    print('DECIMAL DEGREE TO HMS DMS')
    print(f'TRUE VALUES ra: {ra}')
    convert_ra, convert_dec = convert_degree_to_hms_dms(ra_deg, dec_deg)

    assert convert_ra == ra


"""
This code creates the directories and files for the MOA events rereduced by Prof. Ian Bond in the RTModel structure.
"""
import os
import pandas as pd
import merida.moa_rereduced_lightcurve_cls as merida_func

from jasmine.files_organizer.ra_and_dec_conversions import convert_degree_to_hms_dms


def creating_rtmodel_event_directory_with_files(*,
                                                name_prefix_='si01',
                                                folder_to_be_created_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                                 'merida/RTModel_runs/ian',
                                                master_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                                'merida/data/lightcurves_from_ian',
                                                data_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                              'merida/data/lightcurves_from_ian',
                                                limb_darkening_=0.5633,
                                                alternative_lightcurve_version_=None):
    """
    Create ONE directory and files for the moa event following RTModel structure
    :param name_prefix_:
    :param folder_to_be_created_path_:
    :param master_input_folder_path_:
    :param data_input_folder_path_:
    :param limb_darkening_:
    :param alternative_lightcurve_version_: some lightcurves like 12 has an alternative version 'a'
    :return:
    """
    light_curve_master_file = f'{master_input_folder_path_}/stela_list.txt'
    creating_subdirectories(name_prefix_=name_prefix_,
                            folder_where_data_will_be_created_=folder_to_be_created_path_,
                            alternative_lightcurve_version_=alternative_lightcurve_version_)
    creating_data_files(name_prefix_=name_prefix_,
                        master_file_path_=light_curve_master_file,
                        data_input_folder_path_=data_input_folder_path_,
                        limb_darkening_=limb_darkening_,
                        folder_to_be_created_path_=folder_to_be_created_path_,
                        alternative_lightcurve_version_=alternative_lightcurve_version_)
    print(f'{name_prefix_} ALT:{alternative_lightcurve_version_} DONE')
    return None


def creating_subdirectories(*, name_prefix_,
                            folder_where_data_will_be_created_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                           'merida/RTModel_runs/ian',
                            alternative_lightcurve_version_=None):
    """
    Create directories for the moa event following RTModel structure
    :param name_prefix_:
    :param folder_where_data_will_be_created_:
    :param alternative_lightcurve_version_: some lightcurves like 12 has an alternative version 'a'
    """
    if alternative_lightcurve_version_ is not None:
        the_name_prefix = name_prefix_ + alternative_lightcurve_version_
    else:
        the_name_prefix = name_prefix_
    event_folder = f'{folder_where_data_will_be_created_}/{the_name_prefix}'
    data_folder = f'{event_folder}/Data'

    if os.path.isdir(event_folder):
        raise FileExistsError(f'Directory {event_folder} already exists!')
    else:
        os.mkdir(event_folder)
        print(f'Folder {event_folder} created!')

    if os.path.isdir(data_folder):
        raise FileExistsError(f'Directory {data_folder} already exists!')
    else:
        os.mkdir(data_folder)
        print(f'Folder {data_folder} created!')


def creating_data_files(*, name_prefix_,
                        master_file_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                     'merida/data/lightcurves_from_ian/stela_list.txt',
                        data_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                           'merida/RTModel_runs/ian',
                        folder_to_be_created_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                               'merida/RTModel_runs/ian_planet',
                        limb_darkening_=0.5633,
                        alternative_lightcurve_version_=None):
    """
    Create files for the moa event following RTModel structure
    :param name_prefix_:
    :param master_file_path_:
    :param data_input_folder_path_:
    """
    if alternative_lightcurve_version_ is not None:
        the_name_prefix = name_prefix_ + alternative_lightcurve_version_
    else:
        the_name_prefix = name_prefix_
    lc_number = int(name_prefix_[2:])
    event_folder = f'{folder_to_be_created_path_}/{the_name_prefix}'
    data_folder = f'{event_folder}/Data'
    if os.path.exists(f'{data_folder}/MOA.dat'):
        raise FileExistsError(
            f'File MOA.dat already exists in directory {data_folder}')
    else:
        moa_file = open(f'{data_folder}/MOA.dat', 'w')
        moa_file.write('# Mag err HJD-2450000\n')
        the_lightcurve = merida_func.MOAReReducedLightcurve(lightcurve_number_=lc_number,
                                                            data_folder_=data_input_folder_path_,
                                                            master_file_path_=master_file_path_,
                                                            alternative_lightcurve_version=alternative_lightcurve_version_)

        days, magnitudes, magnitudes_errors = the_lightcurve.get_days_magnitudes_errors()
        wanted_columns = pd.concat([magnitudes, magnitudes_errors, days], axis=1)
        wanted_columns.to_csv(moa_file, index=False, mode='a', header=False, sep=' ')
        print(f'File MOA.dat created in {data_folder}.')

        # Write LimbDarkening.txt file
        limb_file = open(f'{data_folder}/LimbDarkening.txt', 'w')
        limb_file.write(f'{limb_darkening_}')

        # Create the file with the event information
        event_ra_deg = the_lightcurve.master_row['RA'].values[0]
        event_dec_deg = the_lightcurve.master_row['DEC'].values[0]
        event_coordinates = convert_degree_to_hms_dms(event_ra_deg, event_dec_deg)
        if os.path.exists(f'{data_folder}/event.coordinates'):
            raise FileExistsError(f'File event.coordinates already exists in directory {data_folder}')
        else:
            with open(f'{data_folder}/event.coordinates', 'w') as f:
                f.write(str(event_coordinates)[1:-1].replace(',', '').replace("'", ''))
            print(f'File event.coordinates created in {data_folder}.')


if __name__ == '__main__':
    # EXAMPLE USAGE
    general_path = '/Users/stela/Documents/Scripts/ai_microlensing'
    # WHERE THE FOLDER WITH THE RTModel STRUCTURE WILL BE CREATED:
    folder_to_be_created = f'{general_path}/merida/RTModel_runs/ian'
    # WHERE THE MASTER FILE IS:
    master_input_folder = f'{general_path}/merida/data/lightcurves_from_ian'
    # WHERE THE DATA FILES ARE:
    data_input_folder = f'{general_path}/merida/data/lightcurves_from_ian'

    # # CREATE ONE
    # creating_one_directory_with_files(name_prefix_=1,
    #                                   folder_to_be_created_path_=folder_to_be_created,
    #                                   master_input_folder_path_=master_input_folder,
    #                                   data_input_folder_path_=data_input_folder,
    #                                   limb_darkening_=0.5633)
    for lc_number in range(11, 32):
        # if lc_number == 4 or lc_number == 8 or lc_number == 10:
        #     continue
        # CREATE ONE
        lc_name_prefix = f'si{lc_number:02}'
        creating_rtmodel_event_directory_with_files(name_prefix_=lc_name_prefix,
                                                    folder_to_be_created_path_=folder_to_be_created,
                                                    master_input_folder_path_=master_input_folder,
                                                    data_input_folder_path_=data_input_folder,
                                                    limb_darkening_=0.5633)
    creating_rtmodel_event_directory_with_files(name_prefix_='si12',
                                                folder_to_be_created_path_=folder_to_be_created,
                                                master_input_folder_path_=master_input_folder,
                                                data_input_folder_path_=data_input_folder,
                                                limb_darkening_=0.5633,
                                                alternative_lightcurve_version_='a')


from pathlib import Path
import pandas as pd
from jasmine import ModelResults

from jasmine.classes_and_files_reader import RTModel_results_cls as rtm_results


def collect_event_summary(event_result, is_ICGS=False):
    """ Collects all parameters from an event result object."""
    data = {"event_name": event_result.event_name}
    categories = ["PS", "PX", "BS", "BO", "LS", "LX", "LO"]
    if is_ICGS:
        categories = ["PS", "PX", "LS", "LX", "LO"]
    for category in categories:
        best_obj = getattr(event_result, f"{category.lower()}_best")
        best_category_model_name = best_obj.name
        data[f"{category.upper()}_name"] = best_category_model_name
        print(event_result.event_folder + '/Models/' + best_category_model_name + '.txt')
        model_results = ModelResults(event_result.event_folder + '/Models/' + best_category_model_name + '.txt')
        # Dynamically add all attributes from model_parameters
        for attr, value in vars(model_results.model_parameters).items():
            data[f"{category.upper()}_{attr}"] = value
        print()
    return data


def loop_through_all_the_folders(folder_with_runs_path_, is_ICGS=False):
    all_event_summaries_ = []
    for folder in Path(folder_with_runs_path_).glob("*"):
        print(folder.name)
        if not folder.is_dir():
            continue
        try:
            event_result = rtm_results.EventResults(folder_with_runs_path_ + folder.name)
            event_result.extract_information_from_nature_file()
            event_result.looking_for_the_names_of_best_model_of_each_category()

            event_data = collect_event_summary(event_result, is_ICGS)

            event_data['complete_classification'] = event_result.complete_classification
            event_data['number_of_final_models'] = event_result.number_of_final_models
            event_data['final_models'] = event_result.final_models
            event_data['found_a_planetary_solution'] = event_result.found_a_planet
            event_data['found_a_2L1S_solution'] = event_result.found_a_2L1S_solution

            all_event_summaries_.append(event_data)

        except Exception as e:
            print(f"Error processing {folder.name}: {e}")

    return all_event_summaries_


def combining_models_with_true_values(true_values_table_path_, models_table_path_):
    """
    This function combines the true values with the models.
    :param true_values_path: Path to the true values CSV file.
    :param models_path: Path to the models CSV file.
    :return: DataFrame with combined data.
    """
    true_values_df = pd.read_csv(true_values_table_path_)
    true_values_df['event_name'] = (
        true_values_df['lcname']
        .str.replace('OMPLDG_croin_cassan/OMPLDG_croin_cassan_', 'event_', regex=False)
        .str.replace('.det.lc', '', regex=False)
    )
    models_df = pd.read_csv(models_table_path_)
    columns_of_interest = [
        "event_name", "Planet_q", "Planet_s", "piEN", "piEE", "u0lens1", "t0lens1",
        "tE_ref", "rho", "alpha", "Planet_inclination", "Planet_orbphase",
        "Planet_period", 'ObsGroup_0_chi2', 'ObsGroup_0_flatchi2'
    ]
    using_true_values = true_values_df[columns_of_interest].copy()

    # Rename columns
    rename_dict = {
        "Planet_q": "mass_ratio",
        "Planet_s": "separation",
        "u0lens1": "u0",
        "t0lens1": "t0",
        "tE_ref": "tE",
        "Planet_inclination": "planet_inclination",
        "Planet_orbphase": "planet_orbphase",
        "Planet_period": "planet_period"
    }

    using_true_values = using_true_values.rename(columns=rename_dict)

    # Rename all columns (except event_name and lcname) with "true_" prefix
    using_true_values = using_true_values.rename(
        columns={col: f"true_{col}" for col in using_true_values.columns if col not in ["event_name"]})
    combined_df = pd.merge(using_true_values, models_df, on='event_name', how='outer')
    # Calculate differences and errors for mass ratio in categories LS, LX, LO:
    categories = ['LS', 'LX', 'LO']
    for category in categories:
        model_prefix = category.lower()
        combined_df[f'{category.upper()}_mass_ratio_diff'] = (combined_df[f'{category.upper()}_mass_ratio'] -
                                                              combined_df['true_mass_ratio']).abs() / combined_df[
                                                                 f'{category.upper()}_mass_ratio_error']
    return combined_df


if __name__ == "__main__":
    # # Define the paths
    # # folder_with_runs_path = '/Users/stela/Documents/teste/sample_rtmodel_v2.4/'
    # folder_with_runs_path = '/gpfsm/dnb34/sishitan/orbital_task/RTModel_runs/269_problematic_events/'
    # # model_output_table_path = '/Users/stela/Documents/Scripts/orbital_task/RTModel_runs/sample_rtmodel_v2.4/all_models_summary.csv'
    # model_output_table_path = '/gpfsm/dnb34/sishitan/orbital_task/RTModel_runs/269_problematic_events/all_models_summary.csv'
    # # master_file = '/Users/stela/Documents/Scripts/orbital_task/data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan.sample.csv'
    # master_file = '/gpfsm/dnb34/sishitan/orbital_task/data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan.sample.csv'
    # # true_and_rtmodel_fits_path = '/Users/stela/Documents/Scripts/orbital_task/RTModel_runs/sample_rtmodel_v2.4/true_and_rtmodel_fits.csv'
    # true_and_rtmodel_fits_path = '/gpfsm/dnb34/sishitan/orbital_task/RTModel_runs/269_problematic_events/true_and_rtmodel_fits.csv'
    # Define the paths
    # folder_with_runs_path = '/gpfsm/dnb34/sishitan/orbital_task/RTModel_runs/sample_rtmodel_v3.1/'
    is_this_a_ICGS_run = True
    root_path = '/Users/stela/Documents/Scripts/orbital_task'
    # root_path = '/gpfsm/dnb34/sishitan/orbital_task'
    folder_with_runs_path = f'{root_path}/RTModel_runs/154_failures_v24_v31/'
    model_output_table_path = f'{root_path}/RTModel_runs/154_failures_v24_v31/all_models_summary.csv'
    master_file = f'{root_path}/data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan.sample.csv'
    true_and_rtmodel_fits_path = f'{root_path}/RTModel_runs/154_failures_v24_v31/ICGS_true_and_rtmodel_fits.csv'

    
    all_event_summaries = loop_through_all_the_folders(folder_with_runs_path, is_ICGS=is_this_a_ICGS_run)
    model_summary_df = pd.DataFrame(all_event_summaries)
    model_summary_df.to_csv(model_output_table_path, index=False)
    print(f"✅ Saved {len(model_summary_df)} events to {model_output_table_path}")

    # Combine with true values
    final_df = combining_models_with_true_values(master_file, model_output_table_path)
    order_of_columns_to_be_saved = [
        'event_name',
        'complete_classification', 'found_a_planetary_solution', 'found_a_2L1S_solution',
        'number_of_final_models', 'final_models',
        'true_mass_ratio', 'true_separation', 'true_piEN', 'true_piEE', 'true_u0', 'true_t0', 'true_tE',
        'true_rho', 'true_alpha', 'true_planet_inclination', 'true_planet_orbphase', 'true_planet_period',
        'true_ObsGroup_0_chi2', 'true_ObsGroup_0_flatchi2',
        'PS_name', 'PS_chi2', 'PS_number_of_parameters', 'PS_u0', 'PS_u0_error', 'PS_tE', 'PS_tE_error', 'PS_t0',
        'PS_t0_error',
        'PS_rho', 'PS_rho_error', 'PS_blends', 'PS_sources', 'PS_blendings', 'PS_blendings_error',
        'PS_baselines', 'PS_baselines_error',
        'PX_name', 'PX_chi2', 'PX_number_of_parameters', 'PX_u0', 'PX_u0_error', 'PX_tE', 'PX_tE_error', 'PX_t0',
        'PX_t0_error',
        'PX_rho', 'PX_rho_error', 'PX_piEN', 'PX_piEN_error', 'PX_piEE', 'PX_piEE_error',
        'PX_blends', 'PX_sources', 'PX_blendings', 'PX_blendings_error', 'PX_baselines', 'PX_baselines_error',
        'BS_name', 'BS_chi2', 'BS_number_of_parameters', 'BS_tE', 'BS_tE_error', 'BS_flux_ratio', 'BS_flux_ratio_error',
        'BS_u01', 'BS_u01_error', 'BS_u02', 'BS_u02_error', 'BS_t01', 'BS_t01_error', 'BS_t02', 'BS_t02_error',
        'BS_rho1', 'BS_rho1_error', 'BS_blends', 'BS_sources', 'BS_blendings', 'BS_blendings_error',
        'BS_baselines', 'BS_baselines_error',
        'BO_name', 'BO_chi2', 'BO_number_of_parameters', 'BO_u01', 'BO_u01_error', 'BO_t01', 'BO_t01_error', 'BO_tE',
        'BO_tE_error',
        'BO_rho1', 'BO_rho1_error', 'BO_xi1', 'BO_xi1_error', 'BO_xi2', 'BO_xi2_error', 'BO_omega', 'BO_omega_error',
        'BO_inc', 'BO_inc_error', 'BO_phi', 'BO_phi_error', 'BO_qs', 'BO_qs_error', 'BO_blends',
        'BO_sources',
        'BO_blendings', 'BO_blendings_error', 'BO_baselines', 'BO_baselines_error',
        'LS_name', 'LS_chi2', 'LS_number_of_parameters', 'LS_separation', 'LS_separation_error', 'LS_mass_ratio',
        'LS_mass_ratio_error', 'LS_mass_ratio_diff',
        'LS_u0', 'LS_u0_error', 'LS_alpha', 'LS_alpha_error', 'LS_rho', 'LS_rho_error', 'LS_tE', 'LS_tE_error',
        'LS_t0', 'LS_t0_error', 'LS_blends', 'LS_sources', 'LS_blendings', 'LS_blendings_error',
        'LS_baselines', 'LS_baselines_error',
        'LX_name', 'LX_chi2', 'LX_number_of_parameters', 'LX_separation', 'LX_separation_error', 'LX_mass_ratio',
        'LX_mass_ratio_error', 'LX_mass_ratio_diff',
        'LX_u0', 'LX_u0_error', 'LX_alpha', 'LX_alpha_error', 'LX_rho', 'LX_rho_error', 'LX_tE', 'LX_tE_error',
        'LX_t0', 'LX_t0_error', 'LX_piEN', 'LX_piEN_error', 'LX_piEE', 'LX_piEE_error', 'LX_blends',
        'LX_sources',
        'LX_blendings', 'LX_blendings_error', 'LX_baselines', 'LX_baselines_error',
        'LO_name', 'LO_chi2', 'LO_number_of_parameters', 'LO_separation', 'LO_separation_error', 'LO_mass_ratio',
        'LO_mass_ratio_error', 'LO_mass_ratio_diff',
        'LO_u0', 'LO_u0_error', 'LO_alpha', 'LO_alpha_error', 'LO_rho', 'LO_rho_error', 'LO_tE',
        'LO_tE_error',
        'LO_t0', 'LO_t0_error', 'LO_piEN', 'LO_piEN_error', 'LO_piEE', 'LO_piEE_error',
        'LO_gamma1', 'LO_gamma1_error', 'LO_gamma2', 'LO_gamma2_error', 'LO_gammaz', 'LO_gammaz_error',
        'LO_blends', 'LO_sources', 'LO_blendings', 'LO_blendings_error', 'LO_baselines',
        'LO_baselines_error']
    if is_this_a_ICGS_run:
        order_of_columns_to_be_saved = [
            'event_name',
            'complete_classification', 'found_a_planetary_solution', 'found_a_2L1S_solution',
            'number_of_final_models', 'final_models',
            'true_mass_ratio', 'true_separation', 'true_piEN', 'true_piEE', 'true_u0', 'true_t0', 'true_tE',
            'true_rho', 'true_alpha', 'true_planet_inclination', 'true_planet_orbphase', 'true_planet_period',
            'true_ObsGroup_0_chi2', 'true_ObsGroup_0_flatchi2',
            'PS_name', 'PS_chi2', 'PS_number_of_parameters', 'PS_u0', 'PS_u0_error', 'PS_tE', 'PS_tE_error', 'PS_t0',
            'PS_t0_error',
            'PS_rho', 'PS_rho_error', 'PS_blends', 'PS_sources', 'PS_blendings', 'PS_blendings_error',
            'PS_baselines', 'PS_baselines_error',
            'PX_name', 'PX_chi2', 'PX_number_of_parameters', 'PX_u0', 'PX_u0_error', 'PX_tE', 'PX_tE_error', 'PX_t0',
            'PX_t0_error',
            'PX_rho', 'PX_rho_error', 'PX_piEN', 'PX_piEN_error', 'PX_piEE', 'PX_piEE_error',
            'PX_blends', 'PX_sources', 'PX_blendings', 'PX_blendings_error', 'PX_baselines', 'PX_baselines_error',
            'LS_name', 'LS_chi2', 'LS_number_of_parameters', 'LS_separation', 'LS_separation_error', 'LS_mass_ratio',
            'LS_mass_ratio_error', 'LS_mass_ratio_diff',
            'LS_u0', 'LS_u0_error', 'LS_alpha', 'LS_alpha_error', 'LS_rho', 'LS_rho_error', 'LS_tE', 'LS_tE_error',
            'LS_t0', 'LS_t0_error', 'LS_blends', 'LS_sources', 'LS_blendings', 'LS_blendings_error',
            'LS_baselines', 'LS_baselines_error',
            'LX_name', 'LX_chi2', 'LX_number_of_parameters', 'LX_separation', 'LX_separation_error', 'LX_mass_ratio',
            'LX_mass_ratio_error', 'LX_mass_ratio_diff',
            'LX_u0', 'LX_u0_error', 'LX_alpha', 'LX_alpha_error', 'LX_rho', 'LX_rho_error', 'LX_tE', 'LX_tE_error',
            'LX_t0', 'LX_t0_error', 'LX_piEN', 'LX_piEN_error', 'LX_piEE', 'LX_piEE_error', 'LX_blends',
            'LX_sources',
            'LX_blendings', 'LX_blendings_error', 'LX_baselines', 'LX_baselines_error',
            'LO_name', 'LO_chi2', 'LO_number_of_parameters', 'LO_separation', 'LO_separation_error', 'LO_mass_ratio',
            'LO_mass_ratio_error', 'LO_mass_ratio_diff',
            'LO_u0', 'LO_u0_error', 'LO_alpha', 'LO_alpha_error', 'LO_rho', 'LO_rho_error', 'LO_tE',
            'LO_tE_error',
            'LO_t0', 'LO_t0_error', 'LO_piEN', 'LO_piEN_error', 'LO_piEE', 'LO_piEE_error',
            'LO_gamma1', 'LO_gamma1_error', 'LO_gamma2', 'LO_gamma2_error', 'LO_gammaz', 'LO_gammaz_error',
            'LO_blends', 'LO_sources', 'LO_blendings', 'LO_blendings_error', 'LO_baselines',
            'LO_baselines_error']
    # check if all columns are present
    missing_columns = [col for col in order_of_columns_to_be_saved if col not in final_df.columns]
    if missing_columns:
        print("❗ Warning: The following columns are missing from the final DataFrame:", missing_columns)
    else:
        print("✅ All dcolumns are present in the DataFrame.")
    final_df = final_df[order_of_columns_to_be_saved]
    # excluding some columns that are not needed
    categories = ['PS', 'PX', 'BS', 'BO', 'LS', 'LX', 'LO']
    if is_this_a_ICGS_run:
        categories = ['PS', 'PX', 'LS', 'LX', 'LO']
    for category in categories:
        final_df = final_df.drop(columns=[f'{category}_blends',
                                         f'{category}_sources', f'{category}_blendings',
                                            f'{category}_blendings_error', f'{category}_baselines',
                                            f'{category}_baselines_error'])
    final_df.to_csv(true_and_rtmodel_fits_path, index=False)
    print("✅ Combined true values with model results and saved to", true_and_rtmodel_fits_path)



import pandas as pd
import numpy as np
import os
from astropy.coordinates import SkyCoord
from astropy.coordinates import solar_system_ephemeris, EarthLocation
from astropy.coordinates import get_body_barycentric, get_body
from astropy.time import Time


def creating_ephemerides_from_lc(*,
                                 satellite_folder_path_='/Users/jmbrashe/VBBOrbital/satellitedir',
                                 pre_ephemeris_from_lightcurve):
    if os.path.isdir(satellite_folder_path_):
        print(f'Directory {satellite_folder_path_} already exists! Continuing to file creation.')
    else:
        os.mkdir(satellite_folder_path_)
        print(f'Folder {satellite_folder_path_} created!')
    solar_system_ephemeris.set('de432s')  # The planetary ephemeris DE432 is
    # based on the ephemeris DE430 (Folkner et al. 2014; default if 'jpl').
    # The main difference between DE432 and DE430 is an update of the estimated orbit of
    # the Pluto system barycenter to support the New Horizons project.

    roman_filters = ['W146', 'Z087', 'K213']
    deldot_str = '{:>11}'.format('0.0000000')  # string of zeros for the delta r column

    for filter_index in range(len(roman_filters)):
        filter_ephemeris_name_ = f'satellite{filter_index + 1}.txt'  # file to be created
        if os.path.exists(f'{satellite_folder_path_}/{filter_ephemeris_name_}'):
            raise FileExistsError(
                f'Ephemerides file {filter_ephemeris_name_} already exists in directory {satellite_folder_path_}')
        else:
            eph_df = pre_ephemeris_from_lightcurve[filter_index]  # dataframe per filter
            eph_df = eph_df.reset_index(drop=True)

            # Load it as ecliptic coordinates
            # barycentrictrueecliptic
            ecliptic_coords = SkyCoord(x=eph_df['X_EQ'] / 3, y=eph_df['Y_EQ'] / 3, z=eph_df['Z_EQ'] / 3,
                                       unit='au', frame='barycentrictrueecliptic', obstime='J2000',
                                       representation_type='cartesian')

            # Transform ecliptic to ICRS (barycentric coordinates)
            icrs_coords = ecliptic_coords.transform_to('icrs')

            # Extract the Cartesian coordinates in AU  + roman location needed to distance to Earth
            icrs_cartesian = icrs_coords.cartesian
            roman_loc = pd.DataFrame({
                'x': icrs_cartesian.x.value,
                'y': icrs_cartesian.y.value,
                'z': icrs_cartesian.z.value,
                'total': np.sqrt(
                    icrs_cartesian.x.value ** 2 + icrs_cartesian.y.value ** 2 + icrs_cartesian.z.value ** 2)})

            # EARTH
            t = Time(eph_df['BJD'], format='jd')
            print(eph_df['BJD'])

            # Get earth location in cartesian ICRS to calculate actual distance ...
            earth_barycentric = get_body_barycentric('earth', t)  # Already returns a SkyCoord object in ICRS
            # Convert to Cartesian representation and ensure units are in AU
            earth_barycentric_au = earth_barycentric.xyz.to('AU')
            # Convert to a pandas DataFrame
            data = {'x': earth_barycentric_au[0].value,
                    'y': earth_barycentric_au[1].value,
                    'z': earth_barycentric_au[2].value,
                    'total': np.sqrt(earth_barycentric_au[0].value ** 2 +
                                     earth_barycentric_au[1].value ** 2 +
                                     earth_barycentric_au[2].value ** 2)}  # Extract values from Quantity
            earth_loc = pd.DataFrame(data)  # Transform to DataFrame if needed

            # Distance calculated by axes
            roman_earth_distance_3D = (roman_loc - earth_loc)
            roman_earth_distance = np.sqrt(roman_earth_distance_3D.x ** 2 +
                                           roman_earth_distance_3D.y ** 2 +
                                           roman_earth_distance_3D.z ** 2)

            # Finalize what columns are needed
            eq_coords = icrs_coords.to_table().to_pandas()
            eq_coords['distance'] = roman_earth_distance
            eq_coords['delta_dot'] = np.zeros(eq_coords.shape[0])
            eq_coords['BJD'] = eph_df['BJD']
            eq_coords = eq_coords[['BJD', 'ra', 'dec', 'distance', 'delta_dot']]

            # Save
            with open(f'{satellite_folder_path_}/{filter_ephemeris_name_}', 'w') as f:
                f.write('$$SOE\n')
                for j in range(eq_coords.shape[0]):
                    line = makeline(eq_coords.iloc[j, 0:4], deldot_str)

                    f.write(line)
                f.write('$$EOE')
            print(f'File {filter_ephemeris_name_} created in {satellite_folder_path_}.')


def ephemeris_data_reader(SubRun, Field, ID, *, folder_path_='data'):
    """
    This function reads the lightcurve data file and returns a pandas dataframe with the Ephemeris XYZ +BJD Data
    :param folder_path_:
    :param data_challenge_lc_number_:
    :param filter_:
    :return: lightcurve_data_df
    """
    fname = f'OMPLDG_croin_cassan_{SubRun}_{Field}_{ID}.det.lc'
    columns = ['Simulation_time', 'measured_relative_flux', 'measured_relative_flux_error', 'true_relative_flux',
               'true_relative_flux_error', 'observatory_code', 'saturation_flag', 'best_single_lens_fit',
               'parallax_shift_t', 'parallax_shift_u', 'BJD', 'source_x', 'source_y', 'lens1_x', 'lens1_y', 'lens2_x',
               'lens2_y', 'X_EQ', 'Y_EQ', 'Z_EQ']
    lightcurve_data_df = pd.read_csv(f'{folder_path_}/{fname}', names=columns, comment='#', sep='\s+')
    eph_list_ = []
    for observatory_code in range(3):
        obs_data = lightcurve_data_df[lightcurve_data_df['observatory_code'] == observatory_code]
        eph_list_.append(obs_data[['BJD', 'X_EQ', 'Y_EQ', 'Z_EQ']])
    return eph_list_


# makes one line of the ephemerides file.
def makeline(dfline, deldot):
    bjd_str = '{:0<17}'.format(dfline['BJD'])
    # print(dfline['BJD'])
    # print(bjd_str)
    ra_str = f"{round(dfline['ra'], 5):5f}"
    ra_split = ra_str.split('.')
    ra_split[1] = '{:0<5}'.format(ra_split[1])
    ra_str = ra_split[0] + '.' + ra_split[1]
    ra_str = '{:>13}'.format(ra_str)
    dec_str = f"{round(dfline['dec'], 5):5f}"
    dec_split = dec_str.split('.')
    # print(dec_split)
    dec_split[1] = '{:0<5}'.format(dec_split[1])
    dec_str = dec_split[0] + '.' + dec_split[1]
    dec_str = '{:>9}'.format(dec_str)
    dist_str = roundton(dfline['distance'], 16)
    line = bjd_str + ' ' + ra_str + ' ' + dec_str + ' ' + dist_str + ' ' + deldot + '\n'
    return line


def roundton(sval, n):
    l_ = list(str(sval))
    if len(l_) <= n:
        s_ = "".join(l_)
        return s_.ljust(n, ' ')
    else:
        if int(l_[n]) >= 5:
            if int(l_[n - 1]) < 9:
                l_[n - 1] = str(int(l_[n - 1]) + 1)
            elif int(l_[n - 1]) == 9:
                j = 1
                while int(l_[n - j]) == 9:
                    l_[n - j] = '0'
                    j += 1
                l_[n - j] = str(int(l_[n - j]) + 1)
            s_ = "".join(l_[0:n])
            return s_
        else:
            s_ = "".join(l_[0:n])
            return s_


if __name__ == "__main__":
    general_path = '/Users/stela/Documents/Scripts/orbital_task/data'
    eph_list = ephemeris_data_reader(0, 163, 1174,
                                     folder_path_=f'{general_path}/gulls_orbital_motion_extracted/OMPLDG_croin_cassan_sample')
    creating_ephemerides_from_lc(satellite_folder_path_=f'{general_path}/satellitedir',
                                 pre_ephemeris_from_lightcurve=eph_list)



import os
import pandas as pd
from tqdm import tqdm

from astropy.coordinates import SkyCoord
from astropy import units

from jasmine.classes_and_files_reader import RMDC26_parquet_cls as RMDC26_cls


def creating_rtmodel_directories(
    parquet_path_="data_hugging_face/RMDC26_Beginner_Tier_test.parquet",
    output_dir_="data",
):
    # Load the full dataframe
    df = pd.read_parquet(parquet_path_)
    df["bjd"] = df["bjd"] - 2_450_000  # Adjust times

    # Check if output directory exists and raise error if it does
    if os.path.exists(output_dir_):
        raise FileExistsError(f"Output directory '{output_dir_}' already exists.")
    os.makedirs(output_dir_)

    # Iterate over unique events
    for event_name in tqdm(df["name"].unique()):
        # Create Event object
        event = RMDC26_cls.Event.from_name(event_name, df)

        event_dir = os.path.join(output_dir_, f"event_{event.name}")

        # Check if output directory exists and raise error if it does
        if os.path.exists(event_dir):
            raise FileExistsError(f"Event directory '{event_dir}' already exists.")
        os.makedirs(event_dir)

        # Iterate over available filters in simple_lightcurve
        for filt, lc_df in event.simple_lightcurve.items():
            save_path = os.path.join(event_dir, f"{filt}_.txt")

            # Write header manually and then the data
            with open(save_path, "w") as f:
                f.write("# Mag err HJD-2450000\n")
                # Ensure the columns are in correct order: mag, mag_err, bjd
                lc_df[["mag", "mag_err", "bjd"]].to_csv(
                    f, index=False, header=False, sep=" "
                )

        # Save events coordinates:
        event_coordinates = SkyCoord(
            event.ra_deg, event.dec_deg, unit=(units.deg, units.deg), obstime="J2000"
        )
        rtmodel_event_coordinates = (
            event_coordinates.to_string("hmsdms")
            .replace("h", ":")
            .replace("m", ":")
            .replace("d", ":")
            .replace("s", "")
        )
        if os.path.exists(f"{event_dir}/event.coordinates"):
            raise FileExistsError(
                f"File event.coordinates already exists in directory {event_dir}"
            )
        else:
            with open(f"{event_dir}/event.coordinates", "w") as f:
                f.write(f"{rtmodel_event_coordinates}")
        print(f"{event_dir} conclude!")


if __name__ == "__main__":
    # Example usage
    parquet_path = "data_hugging_face/RMDC26_Beginner_Tier_test.parquet"
    output_dir = "data"
    creating_rtmodel_directories(parquet_path, output_dir)



import RTModel
import shutil
import glob


def main(general_path,event_name,nprocessors,satellitedir):
    event_path = f'{general_path}/{event_name}'

    rtm = RTModel.RTModel()
    rtm.set_processors(nprocessors=nprocessors)
    rtm.set_event(event_path)
    rtm.archive_run()

    modeltypes = ['PS', 'LS', 'LX', 'LO']
    rtm.set_satellite_dir(satellitedir=satellitedir)
    peak_threshold = 5
    rtm.set_processors(nprocessors=nprocessors)
    rtm.set_event(event_path)
    rtm.set_satellite_dir(satellitedir=satellitedir)
    rtm.config_Reader(otherseasons=0, binning=1000000)
    rtm.config_InitCond(usesatellite=1, peakthreshold=peak_threshold, modelcategories=modeltypes)
    rtm.Reader()
    rtm.InitCond()

    # copy LX initial conditions from archived run
    shutil.copy(f'{event_path}/run-0001/InitCond/InitCondLX.txt', f'{event_path}/InitCond')
    # run fits on LX and LO
    rtm.launch_fits('LX')
    rtm.ModelSelector('LX')
    rtm.launch_fits('LO')
    rtm.ModelSelector('LO')

    #Copy PS and LS results to Models directory
    #I think this is really all we need for the finalizer to work, but we should do a detailed reading of the RTModel code
    #I did a semi-detailed reading only :)
    PS_Models = glob.glob(f'{event_path}/run-0001/Models/PS*')
    for model in PS_Models:
        shutil.copy(model, f'{event_path}/Models/')

    LS_Models = glob.glob(f'{event_path}/run-0001/Models/LS*')
    for model in LS_Models:
        shutil.copy(model, f'{event_path}/Models/')

    rtm.modelcodes = modeltypes #tell RTModel to only look for PS + binary models

    rtm.Finalizer() # Done

if __name__ == '__main__':
    general_path = '/Users/jmbrashe/VBBOrbital/NEWGULLS/MCMC/rerun_plx_OM'
    event_name = 'event_0_128_2350'
    satellitedir = '/Users/jmbrashe/VBBOrbital/NEWGULLS/MCMC/satellitedir'
    nprocessors = 5
    main(general_path=general_path,event_name=event_name,nprocessors=nprocessors,satellitedir=satellitedir)



"""
This code creates the directories and files for the MOA9yr events released Dec2023 in the RTModel structure.
Note the fudge factor was set to 1.0, but should be adjusted accordingly
"""
import os
import pandas as pd
from merida.moa9yr_lightcurve_cls import MOA9yearLightcurve
from merida.metadata_cls import MOA_Lightcurve_Metadata

from jasmine.files_organizer.ra_and_dec_conversions import convert_degree_to_hms_dms


def creating_all_directories_with_files(*,
                                        folder_to_be_created_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                                   'merida/RTModel_runs',
                                        master_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/merida/data',
                                        data_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/qusi_microlensing/data/'
                                                                'microlensing_2M',
                                        limb_darkening_=0.5633,
                                        offset_alternative_=False):
    """
    Create a list of directories and files for the moa9yr event following RTModel structure
    :param folder_to_be_created_path_:
    :param master_input_folder_path_:
    :param data_input_folder_path_:
    :param limb_darkening_:
    :param offset_alternative_:
    :return:
    """
    if os.path.isdir(folder_to_be_created_path_):
        print(f' Found directory {folder_to_be_created_path_}! Continuing.')
    else:
        print(f' Error: {folder_to_be_created_path_} not found.')
    ##########
    # get lc numbers from selected_metadata.csv files created by SIS with the NN outputs that are interesting
    ##########
    # pass dataframe index, subrun, event_id to CreatingDirectories/Files
    light_curve_master_file = f'{master_input_folder_path_}/selected_metadata.csv'
    light_curve_master = pd.read_csv(light_curve_master_file)
    for _, row in light_curve_master.iterrows():
        lc_filename = row['lightcurve_name']
        creating_directories(lightcurve_name_=lc_filename,
                             folder_where_data_will_be_created=folder_to_be_created_path_)
        creating_files(lightcurve_name_=lc_filename,
                       master_file_path_=light_curve_master_file,
                       data_input_folder_path_=data_input_folder_path_,
                       limb_darkening_=limb_darkening_,
                       offset_alternative_=offset_alternative_)
        print(f'{lc_filename} DONE')

    return None


def creating_one_directory_with_files(*,
                                      lc_filename_='gb9-R-8-5-27219',
                                      folder_to_be_created_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                                 'merida/RTModel_runs',
                                      master_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                                'merida/data',
                                      data_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                              'qusi_microlensing/data/microlensing_2M',
                                      limb_darkening_=0.5633,
                                      offset_alternative_=False):
    """
    Create ONE directory and files for the moa9yr event following RTModel structure
    :param lc_filename_:
    :param folder_to_be_created_path_:
    :param master_input_folder_path_:
    :param data_input_folder_path_:
    :param limb_darkening_:
    :param offset_alternative_:
    :return:
    """
    light_curve_master_file = f'{master_input_folder_path_}/selected_metadata.csv'
    creating_directories(lightcurve_name_=lc_filename_,
                         folder_where_data_will_be_created=folder_to_be_created_path_)
    creating_files(lightcurve_name_=lc_filename_,
                   master_file_path_=light_curve_master_file,
                   data_input_folder_path_=data_input_folder_path_,
                   limb_darkening_=limb_darkening_,
                   offset_alternative_=offset_alternative_)
    print(f'{lc_filename_} DONE')

    return None


def creating_directories(*, lightcurve_name_,
                         folder_where_data_will_be_created='/Users/stela/Documents/Scripts/ai_microlensing/'
                                                           'merida/RTModel_runs'):
    """
    Create directories for the moa9yr event following RTModel structure
    """
    event_folder_flux = f'{folder_where_data_will_be_created}/{lightcurve_name_}_flux'
    event_folder_cor_flux = f'{folder_where_data_will_be_created}/{lightcurve_name_}_cor_flux'
    data_folder_flux_ = f'{event_folder_flux}/Data'
    data_folder_cor_flux_ = f'{event_folder_cor_flux}/Data'

    for event_folder in [event_folder_flux, event_folder_cor_flux]:
        if os.path.isdir(event_folder):
            raise FileExistsError(f'Directory {event_folder} already exists!')
        else:
            os.mkdir(event_folder)
            print(f'Folder {event_folder} created!')

    for data_folder in [data_folder_flux_, data_folder_cor_flux_]:
        if os.path.isdir(data_folder):
            raise FileExistsError(f'Directory {data_folder} already exists!')
        else:
            os.mkdir(data_folder)
            print(f'Folder {data_folder} created!')


def creating_files(*, lightcurve_name_,
                   general_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/'
                                        'merida/RTModel_runs',
                   master_file_path_='/Users/stela/Documents/Scripts/ai_microlensing/merida/data/selected_metadata.csv',
                   data_input_folder_path_='/Users/stela/Documents/Scripts/ai_microlensing/qusi_microlensing/data/'
                                           'microlensing_2M',
                   limb_darkening_=0.5633,
                   offset_alternative_=False):
    """
    Create files for the moa9yr event following RTModel structure
    """
    event_folder_flux = f'{general_folder_path_}/{lightcurve_name_}_flux'
    event_folder_cor_flux = f'{general_folder_path_}/{lightcurve_name_}_cor_flux'
    data_folder_flux_ = f'{event_folder_flux}/Data'
    data_folder_cor_flux_ = f'{event_folder_cor_flux}/Data'
    for data_to_be_created_folder_ in [data_folder_flux_, data_folder_cor_flux_]:
        if data_to_be_created_folder_.split('_')[-2] == 'cor':
            corrected = True
        else:
            corrected = False
        if os.path.exists(f'{data_to_be_created_folder_}/MOA.dat'):
            raise FileExistsError(
                f'File MOA.dat already exists in directory {data_to_be_created_folder_}')
        else:
            moa_file = open(f'{data_to_be_created_folder_}/MOA.dat', 'w')
            moa_file.write('# Mag err HJD-2450000\n')
            field = lightcurve_name_.split('-')[0]
            the_lightcurve = MOA9yearLightcurve(lightcurve_name_,
                                                f"{data_input_folder_path_}/{field}/")
            the_lightcurve.get_days_magnitudes_errors(fudge_factor=1.0, offset_alternative=offset_alternative_)
            if corrected:
                wanted_columns = the_lightcurve.lightcurve_dataframe[['cor_magnitudes', 'cor_magnitudes_err', 'HJD']].copy()
                wanted_columns.dropna(inplace=True)
            else:
                wanted_columns = the_lightcurve.lightcurve_dataframe[['magnitudes', 'magnitudes_err', 'HJD']]
            wanted_columns.to_csv(moa_file, index=False, mode='a', header=False, sep=' ')
            print(f'File MOA.dat created in {data_to_be_created_folder_}.')

        # Write LimbDarkening.txt file
        limb_file = open(f'{data_to_be_created_folder_}/LimbDarkening.txt', 'w')
        limb_file.write(f'{limb_darkening_}')

        # Create the file with the event information
        event_metadata = MOA_Lightcurve_Metadata(lightcurve_name_, path_to_metadata=master_file_path_)
        event_ra_deg = event_metadata.ra_j2000
        event_dec_deg = event_metadata.dec_j2000
        event_coordinates = convert_degree_to_hms_dms(event_ra_deg, event_dec_deg)
        if os.path.exists(f'{data_to_be_created_folder_}/event.coordinates'):
            raise FileExistsError(f'File event.coordinates already exists in directory {data_to_be_created_folder_}')
        else:
            with open(f'{data_to_be_created_folder_}/event.coordinates', 'w') as f:
                f.write(str(event_coordinates)[1:-1].replace(',', '').replace("'", ''))
            print(f'File event.coordinates created in {data_to_be_created_folder_}.')


if __name__ == '__main__':
    general_path = '/Users/stela/Documents/Scripts/ai_microlensing'
    folder_to_be_created = f'{general_path}/merida/RTModel_runs'
    master_input_folder = f'{general_path}/merida/data'
    data_input_folder = f'{general_path}/qusi_microlensing/data/microlensing_2M'
    # # CREATE ALL
    # creating_all_directories(folder_to_be_created_path_=folder_to_be_created,
    #                          master_input_folder_path_=master_input_folder,
    #                          data_input_folder_path_=data_input_folder)
    # CREATE ONE
    creating_one_directory_with_files(lc_filename_='gb9-R-8-5-27219',
                                      folder_to_be_created_path_=folder_to_be_created,
                                      master_input_folder_path_=master_input_folder,
                                      data_input_folder_path_=data_input_folder,
                                      limb_darkening_=0.5633,
                                      offset_alternative_=True)


import os
import pandas as pd
import numpy as np
import jasmine.classes_and_files_reader.datachallenge_lightcurve_cls as lc
import jasmine.files_organizer.RTModel_ephemerides_tools as ephtools
from jasmine.constants.limb_darkening_parameters import *
from astropy.coordinates import SkyCoord
from astropy import units


def creating_all_directories(*, event_type_,
                             folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events',
                             master_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data',
                             data_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data'):
    # event_type  should be a list object containing the types of objects in the data challenge you want observed.
    # Choose between 'binary' 'planet' 'CV','single', or you may type 'all'.
    # valid examples: ['binary','planet'], ['all']
    if os.path.isdir(folder_path_):
        print(f' Found directory {folder_path_}! Continuing.')
    else:
        os.mkdir(folder_path_)
        print(f'Folder {folder_path_} created!')
    ##########
    # Check contents of event_type, if necessary prompt for correct inputs
    ##########
    valid_names = ['binary', 'planet', 'cv', 'single', 'all']
    i = 0
    while i < len(event_type_):
        if event_type_[i].lower() not in valid_names:
            name = input(
                f' Event type {event_type_[i]} not recognized.\nPlease re-enter one of these names {valid_names}'
                f' or type exit to close the program.\n')
            if name.lower() == 'exit':
                return None
            else:
                event_type_[i] = name.lower()
                i -= 1
        else:
            event_type_[i] = event_type_[i].lower()
            if event_type_[i] == 'all' and len(event_type_) > 1:
                event_type_ = ['all']
                break
        i += 1
    if 'all' in event_type_:
        event_type_ = valid_names[0:4]  #
    event_type_ = np.unique(event_type_)
    ##########
    # get lc numbers from data master files created by the data_challenge prep module
    ##########
    csv_paths_ = {'binary': f'{data_input_folder_path_}/binary_star.csv',
                  'planet': f'{data_input_folder_path_}/bound_planet.csv',
                  'cv': f'{data_input_folder_path_}/cataclysmic_variables.csv',
                  'single': f'{data_input_folder_path_}/single_lens.csv'}
    data_challenge_lc_number_list = []
    for ev_type in event_type_:
        event_type_master = pd.read_csv(csv_paths_[ev_type])
        data_challenge_lc_number_list.append(event_type_master['data_challenge_lc_number'].values)
    data_challenge_lc_number_list = np.concatenate(data_challenge_lc_number_list)
    print(data_challenge_lc_number_list)
    for dc_lc_num in data_challenge_lc_number_list:
        creating_directories(data_challenge_lc_number_=dc_lc_num,
                             folder_path_=folder_path_)
        creating_files(data_challenge_lc_number_=dc_lc_num,
                       output_folder_path_=folder_path_,
                       master_input_folder_path_=master_input_folder_path_,
                       data_input_folder_path_=data_input_folder_path_)

    return None


def creating_directories(*, data_challenge_lc_number_,
                         folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events'):
    """
    Create directories for the data challenge event following RTModel structure
    """
    event_folder_ = f'{folder_path_}/event_{data_challenge_lc_number_:03}'
    data_folder_ = f'{folder_path_}/event_{data_challenge_lc_number_:03}/Data'
    if os.path.isdir(event_folder_):
        raise FileExistsError(f'Directory {event_folder_} already exists!')
    else:
        os.mkdir(event_folder_)
        print(f'Folder {event_folder_} created!')
    if os.path.isdir(data_folder_):
        raise FileExistsError(f'Directory {data_folder_} already exists!')
    else:
        os.mkdir(data_folder_)
        print(f'Folder {data_folder_} created!')


def creating_files(*, data_challenge_lc_number_,
                   output_folder_path_='/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events',
                   master_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data',
                   data_input_folder_path_='/Users/sishitan/Documents/Scripts/jasmine/data'):
    """
    Create files for the data challenge event following RTModel structure
    """
    data_folder_ = f'{output_folder_path_}/event_{data_challenge_lc_number_:03}/Data'
    filters = ['W149', 'Z087']
    ld_par = [W149, Z087] # Order must match order in FilterToData.txt
    the_lightcurve = lc.LightcurveEventDataChallenge(data_challenge_lc_number_, data_folder=master_input_folder_path_)
    i = 1
    for filter_ in filters:
        lightcurve_data_df = the_lightcurve.lightcurve_data(filter_=filter_, folder_path_=data_input_folder_path_)
        lightcurve_data_df_out = lightcurve_data_df[['Magnitude', 'Error', 'days']]
        if os.path.exists(f'{data_folder_}/Roman{filter_}sat{i}.dat'):
            raise FileExistsError(f'File Roman{filter_}sat{i}.dat already exists in directory {data_folder_}')
        else:
            with open(f'{data_folder_}/Roman{filter_}sat{i}.dat', 'w') as f:
                f.write('# Mag err HJD-2450000\n')
            lightcurve_data_df_out.to_csv(f'{data_folder_}/Roman{filter_}sat{i}.dat', sep=' ',
                                          index=False, header=False, mode='a')
            print(f'File Roman{filter_}sat{i}.dat created in {data_folder_}.')
        i += 1
    # write LimbDarkening.txt file    
    with open(data_folder+'/LimbDarkening.txt','w') as f:
        for mu in ld_par:
            f.write(f'{mu}\n')



    # Create the file with the event information
    event_ra = the_lightcurve.event_info.event_right_ascension__deg
    event_dec = the_lightcurve.event_info.event_declination__deg
    event_coordinates = SkyCoord(event_ra, event_dec, unit=(units.deg, units.deg), obstime='J2000')
    rtmodel_event_coordinates = (event_coordinates.to_string('hmsdms').replace('h', ':').replace('m', ':')
                                 .replace('d', ':').replace('s', ''))
    if os.path.exists(f'{data_folder_}/event.coordinates'):
        raise FileExistsError(f'File event.coordinates already exists in directory {data_folder_}')
    else:
        with open(f'{data_folder_}/event.coordinates', 'w') as f:
            f.write(f'{rtmodel_event_coordinates}')
        print(f'File event.coordinates created in {data_folder_}.')


if __name__ == '__main__':
    folder_to_be_created = '/Users/jmbrashe/VBBOrbital/event_test'
    input_folder = '/Users/jmbrashe/VBBOrbital/data'
    satellite_folder = '/Users/jmbrashe/VBBOrbital/satellitedir'
    creating_all_directories(event_type_=['binary', 'planet'], folder_path_=folder_to_be_created,
                             master_input_folder_path_=input_folder, data_input_folder_path_=input_folder)
    ephtools.creating_ephemerides(satellite_folder_path_=satellite_folder, ephemerides_path_=input_folder)
